@@ -1,15 +1,24 @@
 import ast
-import functools
 import textwrap
 
+from typing import List
+
 from .clike import CLikeTranspiler
-from .declaration_extractor import DeclarationExtractor
-from py2many.tracer import is_list, defined_before, is_class_or_module, is_self_arg
+from .plugins import (
+    ATTR_DISPATCH_TABLE,
+    CLASS_DISPATCH_TABLE,
+    FUNC_DISPATCH_TABLE,
+    MODULE_DISPATCH_TABLE,
+    DISPATCH_MAP,
+    SMALL_DISPATCH_MAP,
+    SMALL_USINGS_MAP,
+)
 
 from py2many.analysis import get_id, is_mutable, is_void_function
+from py2many.clike import class_for_typename
+from py2many.declaration_extractor import DeclarationExtractor
 from py2many.inference import get_inferred_type
-
-from typing import Optional, List
+from py2many.tracer import is_list, defined_before, is_class_or_module, is_self_arg
 
 
 class DartIntegerDivRewriter(ast.NodeTransformer):
@@ -41,6 +50,12 @@ class DartTranspiler(CLikeTranspiler):
         self._container_type_map = self.CONTAINER_TYPE_MAP
         self._default_type = "var"
         self._temp = 0
+        self._dispatch_map = DISPATCH_MAP
+        self._small_dispatch_map = SMALL_DISPATCH_MAP
+        self._small_usings_map = SMALL_USINGS_MAP
+        self._func_dispatch_table = FUNC_DISPATCH_TABLE
+        self._attr_dispatch_table = ATTR_DISPATCH_TABLE
+        self._main_signature_arg_names = ["argv"]
 
     def _get_temp(self):
         self._temp += 1
@@ -61,11 +76,16 @@ class DartTranspiler(CLikeTranspiler):
             del typenames[0]
             del args[0]
 
+        is_python_main = getattr(node, "python_main", False)
+
         typedecls = []
         index = 0
         for i in range(len(args)):
             typename = typenames[i]
             arg = args[i]
+            if is_python_main and arg == "argc":
+                continue
+
             if typename == "T":
                 typename = "T{0}".format(index)
                 typedecls.append(typename)
@@ -76,8 +96,9 @@ class DartTranspiler(CLikeTranspiler):
         if not is_void_function(node):
             if node.returns:
                 typename = self._typename_from_annotation(node, attr="returns")
-                return_type = f"{typename}"
-            else:
+                if typename != self._default_type:
+                    return_type = f"{typename}"
+            if not return_type:
                 return_type = "RT"
                 typedecls.append("RT")
 
@@ -114,6 +135,11 @@ class DartTranspiler(CLikeTranspiler):
 
         value_id = self.visit(node.value)
 
+        if value_id == "sys":
+            if attr == "argv":
+                self._usings.add("dart:io")
+                return "(new List<String>.from([Platform.executable])..addAll(argv))"
+
         if is_list(node.value):
             if node.attr == "append":
                 attr = "add"
@@ -128,66 +154,6 @@ class DartTranspiler(CLikeTranspiler):
 
         return f"{value_id}.{attr}"
 
-    def visit_range(self, node, vargs: List[str]) -> str:
-        start = 0
-        step = 1
-        if len(vargs) == 1:
-            end = vargs[0]
-        elif len(node.args) == 2:
-            start = vargs[0]
-            end = vargs[1]
-        elif len(node.args) == 3:
-            start = vargs[0]
-            end = vargs[1]
-            step = vargs[2]
-        else:
-            raise Exception(
-                "encountered range() call with unknown parameters: range({})".format(
-                    vargs
-                )
-            )
-
-        return f"([for(var i = {start}; i < {end}; i += {step}) i])"
-
-    def visit_print(self, node, vargs: List[str]) -> str:
-        placeholders = []
-        for n in node.args:
-            placeholders.append("%s")
-        self._usings.add("package:sprintf/sprintf.dart")
-        placeholders_str = " ".join(placeholders)
-        vargs_str = ", ".join(vargs)
-        return rf'print(sprintf("{placeholders_str}", [{vargs_str}]))'
-
-    def _dispatch(self, node, fname: str, vargs: List[str]) -> Optional[str]:
-        dispatch_map = {
-            "range": self.visit_range,
-            "xrange": self.visit_range,
-            "print": self.visit_print,
-        }
-
-        if fname in dispatch_map:
-            return dispatch_map[fname](node, vargs)
-
-        def visit_min_max(is_max: bool) -> str:
-            min_max = "max" if is_max else "min"
-            self._usings.add("dart:math")
-            vargs_str = ", ".join(vargs)
-            return f"{min_max}({vargs_str})"
-
-        # small one liners are inlined here as lambdas
-        small_dispatch_map = {
-            "int": lambda: f"{vargs[0]}.toInt()",
-            "str": lambda: f"{vargs[0]}.toString()",
-            "len": lambda: f"{vargs[0]}.length",
-            "floor": lambda: f"{vargs[0]}.floor()",
-            "max": functools.partial(visit_min_max, is_max=True),
-            "min": functools.partial(visit_min_max, is_min=True),
-        }
-
-        if fname in small_dispatch_map:
-            return small_dispatch_map[fname]()
-        return None
-
     def visit_Call(self, node):
         fname = self.visit(node.func)
         vargs = []
@@ -225,10 +191,30 @@ class DartTranspiler(CLikeTranspiler):
     def visit_Compare(self, node):
         left = self.visit(node.left)
         right = self.visit(node.comparators[0])
+        if hasattr(node.comparators[0], "annotation"):
+            self._generic_typename_from_annotation(node.comparators[0])
+            value_type = getattr(
+                node.comparators[0].annotation, "generic_container_type", None
+            )
+            if value_type and value_type[0] == "Dict":
+                right += ".keys"
+
+        if right.endswith(".keys()") or right.endswith(".values()"):
+            right = right[:-2]
+
         if isinstance(node.ops[0], ast.In):
             return "{0}.contains({1})".format(right, left)
         elif isinstance(node.ops[0], ast.NotIn):
             return "!({0}.contains({1}))".format(right, left)
+        elif isinstance(node.ops[0], ast.Eq):
+            if hasattr(node.left, "annotation"):
+                self._generic_typename_from_annotation(node.left)
+                value_type = getattr(
+                    node.left.annotation, "generic_container_type", None
+                )
+                if value_type and value_type[0] == "List":
+                    self._usings.add("package:collection/collection.dart")
+                    return f"DeepCollectionEquality().equals({left}, {right})"
 
         return super().visit_Compare(node)
 
@@ -302,6 +288,16 @@ class DartTranspiler(CLikeTranspiler):
         if ret is not None:
             return ret
 
+        decorators = [get_id(d) for d in node.decorator_list]
+        decorators = [
+            class_for_typename(t, None, self._imported_names) for t in decorators
+        ]
+        for d in decorators:
+            if d in CLASS_DISPATCH_TABLE:
+                ret = CLASS_DISPATCH_TABLE[d](self, node)
+                if ret is not None:
+                    return ret
+
         fields = []
         index = 0
         constructor = ""
@@ -386,21 +382,21 @@ class DartTranspiler(CLikeTranspiler):
             """
         )
 
-    def visit_alias(self, node):
-        return "use {0}".format(node.name)
+    def _import(self, name: str) -> str:
+        return f'import "{name}";'
 
-    def visit_Import(self, node):
-        imports = [self.visit(n) for n in node.names]
-        return "\n".join(i for i in imports if i)
-
-    def visit_ImportFrom(self, node):
-        if node.module in self.IGNORED_MODULE_LIST:
-            return ""
-
-        names = [n.name for n in node.names]
+    def _import_from(self, module_name: str, names: List[str]) -> str:
+        if len(names) == 1:
+            # TODO: make this more generic so it works for len(names) > 1
+            name = names[0]
+            lookup = f"{module_name}.{name}"
+            if lookup in MODULE_DISPATCH_TABLE:
+                dart_module_name, dart_name = MODULE_DISPATCH_TABLE[lookup]
+                dart_module_name = dart_module_name.replace(".", "::")
+                return f"import {dart_module_name};  // {dart_name}"
+        module_name = module_name.replace(".", "::")
         names = ", ".join(names)
-        module_path = node.module.replace(".", "::")
-        return "use {0}::{{{1}}}".format(module_path, names)
+        return f'import "{module_name}";  // {names}'
 
     def visit_List(self, node):
         if len(node.elts) > 0:
@@ -425,7 +421,9 @@ class DartTranspiler(CLikeTranspiler):
             if value == "Tuple":
                 return "({0})".format(index)
             return "{0}<{1}>".format(value, index)
-        return "{0}[{1}]".format(value, index)
+        if getattr(node, "lhs", False):
+            return "{0}[{1}]".format(value, index)
+        return '({0}[{1}] ?? (throw Exception("key not found")))'.format(value, index)
 
     def visit_Index(self, node):
         return self.visit(node.value)
@@ -447,15 +445,10 @@ class DartTranspiler(CLikeTranspiler):
             return elts
         return "({0})".format(elts)
 
-    def visit_unsupported_body(self, name, body):
-        buf = ["let {0} = {{ //unsupported".format(name)]
-        buf += [self.visit(n) for n in body]
-        buf.append("};")
-        return buf
-
     def visit_Raise(self, node):
-        exc = self.visit(node.exc)
-        return f"throw new {exc};"
+        if node.exc is not None:
+            return "throw new {};".format(self.visit(node.exc))
+        return "throw new Exception();"
 
     def visit_Try(self, node):
         buf = []
@@ -522,17 +515,13 @@ class DartTranspiler(CLikeTranspiler):
             value = self.visit(node.value)
             return f"{target} = {value};"
 
-        definition = node.scopes.find(target.id)
+        definition = node.scopes.parent_scopes.find(get_id(target))
+        if definition is None:
+            definition = node.scopes.find(get_id(target))
         if isinstance(target, ast.Name) and defined_before(definition, node):
             target = self.visit(target)
             value = self.visit(node.value)
             return f"{target} = {value};"
-        elif isinstance(node.value, ast.List):
-            elements = [self.visit(e) for e in node.value.elts]
-            elements = ", ".join(elements)
-            target = self.visit(target)
-
-            return f"{kw} {target} = [{elements}];"
         else:
             typename = self._typename_from_annotation(target)
             target = self.visit(target)
@@ -550,27 +539,6 @@ class DartTranspiler(CLikeTranspiler):
         target = node.targets[0]
         return "{0}.drop()".format(self.visit(target))
 
-    def visit_With(self, node):
-        buf = []
-
-        with_statement = "// with!("
-        for i in node.items:
-            if i.optional_vars:
-                with_statement += "{0} as {1}, ".format(
-                    self.visit(i.context_expr), self.visit(i.optional_vars)
-                )
-            else:
-                with_statement += "{0}, ".format(self.visit(i.context_expr))
-        with_statement = with_statement[:-2] + ") //unsupported\n{"
-        buf.append(with_statement)
-
-        for n in node.body:
-            buf.append(self.visit(n))
-
-            buf.append("}")
-
-        return "\n".join(buf)
-
     def visit_Await(self, node):
         return "await!({0})".format(self.visit(node.value))
 
@@ -587,9 +555,6 @@ class DartTranspiler(CLikeTranspiler):
             buf.append('println("{{:?}}",{0})'.format(value))
         return "\n".join(buf)
 
-    def visit_DictComp(self, node):
-        return "DictComp /*unimplemented()*/"
-
     def visit_GeneratorExp(self, node):
         elt = self.visit(node.elt)
         generator = node.generators[0]
