@@ -1,14 +1,24 @@
 import ast
-import functools
 import textwrap
 
-from typing import Optional, List
+from typing import List
 
 from .clike import CLikeTranspiler
-from .declaration_extractor import DeclarationExtractor
 from .inference import get_inferred_go_type
+from .plugins import (
+    ATTR_DISPATCH_TABLE,
+    CLASS_DISPATCH_TABLE,
+    FUNC_DISPATCH_TABLE,
+    MODULE_DISPATCH_TABLE,
+    DISPATCH_MAP,
+    SMALL_DISPATCH_MAP,
+    SMALL_USINGS_MAP,
+)
 
-from py2many.analysis import get_id, is_global, is_void_function
+from py2many.analysis import IGNORED_MODULE_SET, get_id, is_global, is_void_function
+from py2many.clike import _AUTO_INVOKED, class_for_typename
+from py2many.declaration_extractor import DeclarationExtractor
+from py2many.exceptions import AstClassUsedBeforeDeclaration, AstCouldNotInfer
 from py2many.rewriters import capitalize_first, rename, camel_case
 from py2many.tracer import is_list, defined_before, is_class_or_module, is_enum
 
@@ -20,12 +30,14 @@ class GoMethodCallRewriter(ast.NodeTransformer):
         if isinstance(fname, ast.Attribute):
             if is_list(node.func.value) and fname.attr == "append":
                 needs_assign = True
-            if get_id(fname.value):
-                node0 = ast.Name(id=get_id(fname.value), lineno=node.lineno)
-            else:
-                node0 = fname.value
-            node.args = [node0] + node.args
-            node.func = ast.Name(id=fname.attr, lineno=node.lineno, ctx=fname.ctx)
+            value_id = get_id(fname.value)
+            if value_id not in IGNORED_MODULE_SET:
+                if value_id:
+                    node0 = ast.Name(id=get_id(fname.value), lineno=node.lineno)
+                else:
+                    node0 = fname.value
+                node.args = [node0] + node.args
+                node.func = ast.Name(id=fname.attr, lineno=node.lineno, ctx=fname.ctx)
         if needs_assign:
             ret = ast.Assign(
                 targets=[ast.Name(id=fname.value.id, lineno=node.lineno)],
@@ -125,6 +137,11 @@ class GoTranspiler(CLikeTranspiler):
         super().__init__()
         self._default_type = None
         self._container_type_map = self.CONTAINER_TYPE_MAP
+        self._dispatch_map = DISPATCH_MAP
+        self._small_dispatch_map = SMALL_DISPATCH_MAP
+        self._small_usings_map = SMALL_USINGS_MAP
+        self._func_dispatch_table = FUNC_DISPATCH_TABLE
+        self._attr_dispatch_table = ATTR_DISPATCH_TABLE
 
     def headers(self, meta):
         return "\n".join(self._headers)
@@ -156,6 +173,7 @@ class GoTranspiler(CLikeTranspiler):
         for i in range(len(args)):
             typename = typenames[i]
             arg = args[i]
+
             if typename == "T":
                 typename = "T{0} any".format(index)
                 typedecls.append(typename)
@@ -165,11 +183,13 @@ class GoTranspiler(CLikeTranspiler):
         return_type = ""
         if not is_void_function(node):
             if node.returns:
-                typename = self._typename_from_annotation(node, attr="returns")
-                return_type = f" {typename}"
+                try:
+                    typename = self._typename_from_annotation(node, attr="returns")
+                    return_type = f" {typename}"
+                except AstCouldNotInfer:
+                    pass
             else:
-                return_type = " RT"
-                typedecls.append("RT")
+                return_type = " interface{}"
 
         template = ""
         if len(typedecls) > 0:
@@ -188,7 +208,11 @@ class GoTranspiler(CLikeTranspiler):
                     fndef = scope
                     break
             if fndef:
-                return_type = self._typename_from_annotation(fndef, attr="returns")
+                return_type = None
+                try:
+                    return_type = self._typename_from_annotation(fndef, attr="returns")
+                except AstCouldNotInfer:
+                    pass
                 value_type = get_inferred_go_type(node.value)
                 if return_type != value_type and value_type is not None:
                     return f"return {return_type}({ret})"
@@ -220,6 +244,11 @@ class GoTranspiler(CLikeTranspiler):
 
         value_id = self.visit(node.value)
 
+        if value_id == "sys":
+            if attr == "argv":
+                self._usings.add('"os"')
+                return "os.Args"
+
         if not value_id:
             value_id = ""
 
@@ -231,67 +260,12 @@ class GoTranspiler(CLikeTranspiler):
 
         return f"{value_id}.{attr}"
 
-    def visit_range(self, node, vargs: List[str]) -> str:
-        self._usings.add('iter "github.com/hgfischer/go-iter"')
-        if len(node.args) == 1:
-            return f"iter.NewIntSeq(iter.Start(0), iter.Stop({vargs[0]})).All()"
-        elif len(node.args) == 2:
-            return (
-                f"iter.NewIntSeq(iter.Start({vargs[0]}), iter.Stop({vargs[1]})).All()"
-            )
-        elif len(node.args) == 3:
-            return f"iter.NewIntSeq(iter.Start({vargs[0]}), iter.Stop({vargs[1]}), iter.Step({vargs[2]})).All()"
-
-        raise Exception(
-            "encountered range() call with unknown parameters: range({})".format(vargs)
-        )
-
-    def visit_print(self, node, vargs: List[str]) -> str:
-        placeholders = []
-        for n in node.args:
-            placeholders.append("%v")
-        self._usings.add('"fmt"')
-        placeholders_str = " ".join(placeholders)
-        vargs_str = ", ".join(vargs)
-        return f'fmt.Printf("{placeholders_str}\\n",{vargs_str})'
-
-    def _dispatch(self, node, fname: str, vargs: List[str]) -> Optional[str]:
-        dispatch_map = {
-            "range_": self.visit_range,
-            "xrange": self.visit_range,
-            "print": self.visit_print,
-        }
-
-        if fname in dispatch_map:
-            return dispatch_map[fname](node, vargs)
-
-        def visit_min_max(is_max: bool) -> str:
-            min_max = "math.Max" if is_max else "math.Min"
-            self._usings.add('"math"')
-            vargs_str = ", ".join(vargs)
-            return f"{min_max}({vargs_str})"
-
-        def visit_floor():
-            self._usings.add('"math"')
-            return f"math.Floor({vargs[0]})"
-
-        # small one liners are inlined here as lambdas
-        small_dispatch_map = {
-            "str": lambda: f"String({vargs[0]})",
-            "max": functools.partial(visit_min_max, is_max=True),
-            "min": functools.partial(visit_min_max, is_min=True),
-            "floor": visit_floor,
-        }
-        if fname in small_dispatch_map:
-            return small_dispatch_map[fname]()
-        return None
-
     def _visit_struct_literal(self, node, fname: str, fndef: ast.ClassDef):
         vargs = []  # visited args
         if not hasattr(fndef, "declarations"):
-            raise Exception("Missing declarations")
+            raise AstClassUsedBeforeDeclaration(fndef, node)
         if node.args:
-            for arg, decl in zip(node.args, fndef.declaration.keys()):
+            for arg, decl in zip(node.args, fndef.declarations.keys()):
                 arg = self.visit(arg)
                 vargs += [f"{decl}: {arg}"]
         if node.keywords:
@@ -382,6 +356,10 @@ class GoTranspiler(CLikeTranspiler):
     def visit_Name(self, node):
         if node.id == "None":
             return "None"
+        elif node.id == "StringsContains":
+            # TODO: move to plugins
+            self._usings.add('"strings"')
+            return "strings.Contains"
         else:
             return super().visit_Name(node)
 
@@ -442,6 +420,16 @@ class GoTranspiler(CLikeTranspiler):
         if ret is not None:
             return ret
 
+        decorators = [get_id(d) for d in node.decorator_list]
+        decorators = [
+            class_for_typename(t, None, self._imported_names) for t in decorators
+        ]
+        for d in decorators:
+            if d in CLASS_DISPATCH_TABLE:
+                ret = CLASS_DISPATCH_TABLE[d](self, node)
+                if ret is not None:
+                    return ret
+
         fields = []
         index = 0
         for declaration, typename in declarations.items():
@@ -454,23 +442,17 @@ class GoTranspiler(CLikeTranspiler):
             if isinstance(b, ast.FunctionDef):
                 b.self_type = node.name
 
-        if node.is_dataclass:
-            fields = "\n".join(fields)
-            body = [self.visit(b) for b in node.body]
-            body = "\n".join(body)
-            return f"type {node.name} struct {{\n{fields}\n}}\n{body}\n"
-        else:
-            fields = "\n".join(fields)
-            body = [self.visit(b) for b in node.body]
-            body = "\n".join(body)
-            return f"class {node.name} {{\n{fields}\n\n {body}\n}}\n"
+        fields = "\n".join(fields)
+        body = [self.visit(b) for b in node.body]
+        body = "\n".join(body)
+        return f"type {node.name} struct {{\n{fields}\n}}\n{body}\n"
 
     def _visit_enum(self, node, typename, members):
         ret = f"type {node.name} {typename}\n\n"
         fields = []
         for i, (member, var) in enumerate(members):
             typename = f" {node.name}" if i == 0 else ""
-            if var == "auto()":
+            if var == _AUTO_INVOKED:
                 fields.append(f"{member}{typename} = iota")
             else:
                 fields.append(f"{member}{typename} = {var}")
@@ -481,7 +463,7 @@ class GoTranspiler(CLikeTranspiler):
         members = []
         for i, (member, var) in enumerate(node.class_assignments.items()):
             var = self.visit(var)
-            if var == "auto()":
+            if var == _AUTO_INVOKED:
                 members.append((member, "iota"))
             else:
                 members.append((member, var))
@@ -491,7 +473,7 @@ class GoTranspiler(CLikeTranspiler):
         members = []
         for i, (member, var) in enumerate(node.class_assignments.items()):
             var = self.visit(var)
-            if var == "auto()":
+            if var == _AUTO_INVOKED:
                 members.append((member, "1 << iota"))
             else:
                 members.append((member, var))
@@ -504,21 +486,20 @@ class GoTranspiler(CLikeTranspiler):
             members.append((member, var))
         return self._visit_enum(node, "string", members)
 
-    def visit_alias(self, node):
-        return "use {0}".format(node.name)
+    def _import(self, name: str) -> str:
+        return f'import ("{name}")'
 
-    def visit_Import(self, node):
-        imports = [self.visit(n) for n in node.names]
-        return "\n".join(i for i in imports if i)
-
-    def visit_ImportFrom(self, node):
-        if node.module in self.IGNORED_MODULE_LIST:
-            return ""
-
-        names = [n.name for n in node.names]
-        names = ", ".join(names)
-        module_path = node.module.replace(".", "::")
-        return "use {0}::{{{1}}}".format(module_path, names)
+    def _import_from(self, module_name: str, names: List[str]) -> str:
+        if len(names) == 1:
+            # TODO: make this more generic so it works for len(names) > 1
+            name = names[0]
+            lookup = f"{module_name}.{name}"
+            if lookup in MODULE_DISPATCH_TABLE:
+                go_module_name, go_name = MODULE_DISPATCH_TABLE[lookup]
+                go_module_name = go_module_name.replace(".", "/")
+                return f'import ("{go_module_name}/{go_name}")'
+        module_name = module_name.replace(".", "/")
+        return "\n".join([f'import ("{module_name}/{name}")' for name in names])
 
     def visit_List(self, node):
         _ = self._typename_from_annotation(node)
@@ -582,21 +563,15 @@ class GoTranspiler(CLikeTranspiler):
             return elts
         return "{0}".format(elts)
 
-    def visit_unsupported_body(self, name, body):
-        buf = ["let {0} = {{ //unsupported".format(name)]
-        buf += [self.visit(n) for n in body]
-        buf.append("};")
-        return buf
-
     def visit_Try(self, node, finallybody=None):
-        buf = self.visit_unsupported_body("try_dummy", node.body)
+        buf = self.visit_unsupported_body(node, "try_dummy", node.body)
 
         for handler in node.handlers:
             buf += self.visit(handler)
         # buf.append("\n".join(excepts));
 
         if finallybody:
-            buf += self.visit_unsupported_body("finally_dummy", finallybody)
+            buf += self.visit_unsupported_body(node, "finally_dummy", finallybody)
 
         return "\n".join(buf)
 
@@ -605,7 +580,7 @@ class GoTranspiler(CLikeTranspiler):
         if node.type:
             exception_type = self.visit(node.type)
         name = "except!({0})".format(exception_type)
-        body = self.visit_unsupported_body(name, node.body)
+        body = self.visit_unsupported_body(node, name, node.body)
         return body
 
     def visit_Assert(self, node):
@@ -669,15 +644,19 @@ class GoTranspiler(CLikeTranspiler):
                 value, typename, left_annotation, right_annotation
             )
 
-        definition = node.scopes.find(target.id)
+        definition = node.scopes.parent_scopes.find(get_id(target))
+        if definition is None:
+            definition = node.scopes.find(get_id(target))
         if isinstance(target, ast.Name) and defined_before(definition, node):
             return f"{target_str} = {value}"
         else:
             if typename is not None:
                 # Dummy assign to silence the compiler on unused vars
                 maybe_tail = ""
-                if target_str.startswith("_"):
+                if target_str.startswith("_") and target_str != "_":
                     maybe_tail = f"\n_ = {target_str}"
+                if target_str == "_":
+                    return f"{target_str} = {value}"
                 return f"var {target_str} {typename} = {value}{maybe_tail}"
 
             if is_global(node):
@@ -695,27 +674,6 @@ class GoTranspiler(CLikeTranspiler):
         # specifying the exception.
         return "raise!(); //unsupported"
 
-    def visit_With(self, node):
-        buf = []
-
-        with_statement = "// with!("
-        for i in node.items:
-            if i.optional_vars:
-                with_statement += "{0} as {1}, ".format(
-                    self.visit(i.context_expr), self.visit(i.optional_vars)
-                )
-            else:
-                with_statement += "{0}, ".format(self.visit(i.context_expr))
-        with_statement = with_statement[:-2] + ") //unsupported\n{"
-        buf.append(with_statement)
-
-        for n in node.body:
-            buf.append(self.visit(n))
-
-            buf.append("}")
-
-        return "\n".join(buf)
-
     def visit_Await(self, node):
         return "await!({0})".format(self.visit(node.value))
 
@@ -733,9 +691,6 @@ class GoTranspiler(CLikeTranspiler):
             buf.append('fmt.Printf("%v\n",{0})'.format(value))
         return "\n".join(buf)
 
-    def visit_DictComp(self, node):
-        return "DictComp /*unimplemented()*/"
-
     def visit_GeneratorExp(self, node):
         elt = self.visit(node.elt)
         generator = node.generators[0]
