@@ -2,17 +2,25 @@ import ast
 import textwrap
 
 from .clike import CLikeTranspiler
-from .declaration_extractor import DeclarationExtractor
-from py2many.tracer import is_list, defined_before, is_class_or_module, is_enum
+from .plugins import (
+    ATTR_DISPATCH_TABLE,
+    CLASS_DISPATCH_TABLE,
+    FUNC_DISPATCH_TABLE,
+    MODULE_DISPATCH_TABLE,
+    DISPATCH_MAP,
+    SMALL_DISPATCH_MAP,
+    SMALL_USINGS_MAP,
+)
 
 from py2many.analysis import get_id, is_void_function
-from typing import Optional, List, Tuple
+from py2many.declaration_extractor import DeclarationExtractor
+from py2many.clike import _AUTO_INVOKED, class_for_typename
+from py2many.tracer import is_list, defined_before, is_class_or_module, is_enum
+
+from typing import List, Tuple
 
 
 class JuliaMethodCallRewriter(ast.NodeTransformer):
-    def __init__(self):
-        super().__init__()
-
     def visit_Call(self, node):
         fname = node.func
         if isinstance(fname, ast.Attribute):
@@ -42,8 +50,13 @@ class JuliaTranspiler(CLikeTranspiler):
     def __init__(self):
         super().__init__()
         self._headers = set([])
-        self._default_type = None
+        self._default_type = ""
         self._container_type_map = self.CONTAINER_TYPE_MAP
+        self._dispatch_map = DISPATCH_MAP
+        self._small_dispatch_map = SMALL_DISPATCH_MAP
+        self._small_usings_map = SMALL_USINGS_MAP
+        self._func_dispatch_table = FUNC_DISPATCH_TABLE
+        self._attr_dispatch_table = ATTR_DISPATCH_TABLE
 
     def usings(self):
         usings = sorted(list(set(self._usings)))
@@ -63,6 +76,11 @@ class JuliaTranspiler(CLikeTranspiler):
             return "false"
         elif node.value is None:
             return "nothing"
+        elif isinstance(node.value, complex):
+            str_value = str(node.value)
+            return (
+                str_value.replace("j", "im") if str_value.endswith("j") else str_value
+            )
         else:
             return super().visit_Constant(node)
 
@@ -74,6 +92,8 @@ class JuliaTranspiler(CLikeTranspiler):
         typedecls = []
         index = 0
 
+        is_python_main = getattr(node, "python_main", False)
+
         if len(typenames) and typenames[0] == None and hasattr(node, "self_type"):
             typenames[0] = node.self_type
 
@@ -102,7 +122,7 @@ class JuliaTranspiler(CLikeTranspiler):
         args = ", ".join(args_list)
         funcdef = f"function {node.name}{template}({args}){return_type}"
         maybe_main = ""
-        if getattr(node, "python_main", False):
+        if is_python_main:
             maybe_main = "\nmain()"
         return f"{funcdef}\n{body}\nend\n{maybe_main}"
 
@@ -134,6 +154,10 @@ class JuliaTranspiler(CLikeTranspiler):
         if not value_id:
             value_id = ""
 
+        if value_id == "sys":
+            if attr == "argv":
+                return "append!([PROGRAM_FILE], ARGS)"
+
         if is_enum(value_id, node.scopes):
             return f"{value_id}.{attr}"
 
@@ -154,38 +178,10 @@ class JuliaTranspiler(CLikeTranspiler):
             "encountered range() call with unknown parameters: range({})".format(vargs)
         )
 
-    def visit_print(self, node, vargs: List[str]) -> str:
+    def _visit_print(self, node, vargs: List[str]) -> str:
         args = ", ".join(vargs)
         return f'println(join([{args}], " "))'
 
-    def _dispatch(self, node, fname: str, vargs: List[str]) -> Optional[str]:
-        dispatch_map = {
-            "range": self.visit_range,
-            "xrange": self.visit_range,
-            "print": self.visit_print,
-        }
-
-        if fname in dispatch_map:
-            return dispatch_map[fname](node, vargs)
-
-        def visit_cast_int() -> str:
-            arg_type = self._typename_from_annotation(node.args[0])
-            if arg_type is not None and arg_type.startswith("Float"):
-                return f"Int64(floor({vargs[0]}))"
-            return f"Int64({vargs[0]})"
-
-        # small one liners are inlined here as lambdas
-        small_dispatch_map = {
-            "int": visit_cast_int,
-            "str": lambda: f"string({vargs[0]})",
-            "len": lambda: f"length({vargs[0]})",
-            # ::Int64 below is a hack to pass comb_sort.jl. Need a better solution
-            "floor": lambda: f"Int64(floor({vargs[0]}))",
-        }
-        if fname in small_dispatch_map:
-            return small_dispatch_map[fname]()
-        return None
-
     def visit_Call(self, node):
         fname = self.visit(node.func)
         fndef = node.scopes.find(fname)
@@ -228,8 +224,28 @@ class JuliaTranspiler(CLikeTranspiler):
         return "" + super().visit_Str(node) + ""
 
     def visit_Bytes(self, node):
-        bytes_str = "{0}".format(node.s)
-        return bytes_str.replace("'", '"')  # replace single quote with double quote
+        bytes_str = node.s
+        bytes_str = bytes_str.replace(b'"', b'\\"')
+        return 'b"' + bytes_str.decode("ascii", "backslashreplace") + '"'
+
+    def visit_Compare(self, node):
+        left = self.visit(node.left)
+        right = self.visit(node.comparators[0])
+
+        if hasattr(node.comparators[0], "annotation"):
+            self._generic_typename_from_annotation(node.comparators[0])
+            value_type = getattr(
+                node.comparators[0].annotation, "generic_container_type", None
+            )
+            if value_type and value_type[0] == "Dict":
+                right = f"keys({right})"
+
+        if isinstance(node.ops[0], ast.In):
+            return "{0} in {1}".format(left, right)
+        elif isinstance(node.ops[0], ast.NotIn):
+            return "{0} not in {1}".format(left, right)
+
+        return super().visit_Compare(node)
 
     def visit_Name(self, node):
         if node.id == "None":
@@ -305,13 +321,23 @@ class JuliaTranspiler(CLikeTranspiler):
         if ret is not None:
             return ret
 
+        decorators = [get_id(d) for d in node.decorator_list]
+        decorators = [
+            class_for_typename(t, None, self._imported_names) for t in decorators
+        ]
+        for d in decorators:
+            if d in CLASS_DISPATCH_TABLE:
+                ret = CLASS_DISPATCH_TABLE[d](self, node)
+                if ret is not None:
+                    return ret
+
         fields = []
         index = 0
         for declaration, typename in declarations.items():
             if typename == None:
                 typename = "ST{0}".format(index)
                 index += 1
-            fields.append(f"{declaration}::{typename}".format(declaration, typename))
+            fields.append(f"{declaration}::{typename}")
 
         fields = "\n".join(fields)
         struct_def = f"struct {node.name}\n{fields}\nend\n"
@@ -344,7 +370,7 @@ class JuliaTranspiler(CLikeTranspiler):
         fields = []
         for i, (member, var) in enumerate(node.class_assignments.items()):
             var = self.visit(var)
-            if var == "auto()":
+            if var == _AUTO_INVOKED:
                 var = f'"{member}"'
             fields.append((member, var))
         return self._visit_enum(node, "String", fields)
@@ -353,7 +379,7 @@ class JuliaTranspiler(CLikeTranspiler):
         fields = []
         for i, (member, var) in enumerate(node.class_assignments.items()):
             var = self.visit(var)
-            if var == "auto()":
+            if var == _AUTO_INVOKED:
                 var = i
             fields.append((member, var))
         return self._visit_enum(node, "Int64", fields)
@@ -362,26 +388,26 @@ class JuliaTranspiler(CLikeTranspiler):
         fields = []
         for i, (member, var) in enumerate(node.class_assignments.items()):
             var = self.visit(var)
-            if var == "auto()":
+            if var == _AUTO_INVOKED:
                 var = 1 << i
             fields.append((member, var))
         return self._visit_enum(node, "Int64", fields)
 
-    def visit_alias(self, node):
-        return "use {0}".format(node.name)
+    def _import(self, name: str) -> str:
+        return f"import {name}"
 
-    def visit_Import(self, node):
-        imports = [self.visit(n) for n in node.names]
-        return "\n".join(i for i in imports if i)
-
-    def visit_ImportFrom(self, node):
-        if node.module in self.IGNORED_MODULE_LIST:
-            return ""
-
-        names = [n.name for n in node.names]
+    def _import_from(self, module_name: str, names: List[str]) -> str:
+        if len(names) == 1:
+            # TODO: make this more generic so it works for len(names) > 1
+            name = names[0]
+            lookup = f"{module_name}.{name}"
+            if lookup in MODULE_DISPATCH_TABLE:
+                jl_module_name, jl_name = MODULE_DISPATCH_TABLE[lookup]
+                jl_module_name = jl_module_name.replace(".", "::")
+                return f"using {jl_module_name}: {jl_name}"
+        module_name = module_name.replace(".", "::")
         names = ", ".join(names)
-        module_path = node.module.replace(".", "::")
-        return "use {0}::{{{1}}}".format(module_path, names)
+        return f"using {module_name}: {names}"
 
     def visit_List(self, node):
         elements = [self.visit(e) for e in node.elts]
@@ -410,12 +436,13 @@ class JuliaTranspiler(CLikeTranspiler):
             return "{0}{{{1}}}".format(value, index)
         # TODO: optimize this. We need to compute value_type once per definition
         self._generic_typename_from_annotation(node.value)
-        value_type = getattr(node.value.annotation, "generic_container_type", None)
-        if value_type is not None and value_type[0] == "List":
-            # Julia array indices start at 1
-            return "{0}[{1} + 1]".format(value, index)
-        else:
-            return "{0}[{1}]".format(value, index)
+        if hasattr(node.value, "annotation"):
+            value_type = getattr(node.value.annotation, "generic_container_type", None)
+            if value_type is not None and value_type[0] == "List":
+                # Julia array indices start at 1
+                return "{0}[{1} + 1]".format(value, index)
+
+        return "{0}[{1}]".format(value, index)
 
     def visit_Index(self, node):
         return self.visit(node.value)
@@ -437,21 +464,15 @@ class JuliaTranspiler(CLikeTranspiler):
             return elts
         return "({0})".format(elts)
 
-    def visit_unsupported_body(self, name, body):
-        buf = ["let {0} = {{ //unsupported".format(name)]
-        buf += [self.visit(n) for n in body]
-        buf.append("}")
-        return buf
-
     def visit_Try(self, node, finallybody=None):
-        buf = self.visit_unsupported_body("try_dummy", node.body)
+        buf = self.visit_unsupported_body(node, "try_dummy", node.body)
 
         for handler in node.handlers:
             buf += self.visit(handler)
         # buf.append("\n".join(excepts));
 
         if finallybody:
-            buf += self.visit_unsupported_body("finally_dummy", finallybody)
+            buf += self.visit_unsupported_body(node, "finally_dummy", finallybody)
 
         return "\n".join(buf)
 
@@ -460,7 +481,7 @@ class JuliaTranspiler(CLikeTranspiler):
         if node.type:
             exception_type = self.visit(node.type)
         name = "except!({0})".format(exception_type)
-        body = self.visit_unsupported_body(name, node.body)
+        body = self.visit_unsupported_body(node, name, node.body)
         return body
 
     def visit_Assert(self, node):
@@ -498,14 +519,13 @@ class JuliaTranspiler(CLikeTranspiler):
                 value = "None"
             return "{0} = {1}".format(target, value)
 
-        definition = node.scopes.find(target.id)
+        definition = node.scopes.parent_scopes.find(get_id(target))
+        if definition is None:
+            definition = node.scopes.find(get_id(target))
         if isinstance(target, ast.Name) and defined_before(definition, node):
             target_str = self.visit(target)
             value = self.visit(node.value)
             return f"{target_str} = {value};"
-        elif isinstance(node.value, ast.List):
-            elements = [self.visit(e) for e in node.value.elts]
-            return "{0} = [{1}]".format(self.visit(target), ", ".join(elements))
         else:
             target = self.visit(target)
             value = self.visit(node.value)
@@ -522,27 +542,6 @@ class JuliaTranspiler(CLikeTranspiler):
         # specifying the exception.
         return "raise!() # unsupported"
 
-    def visit_With(self, node):
-        buf = []
-
-        with_statement = "// with!("
-        for i in node.items:
-            if i.optional_vars:
-                with_statement += "{0} as {1}, ".format(
-                    self.visit(i.context_expr), self.visit(i.optional_vars)
-                )
-            else:
-                with_statement += "{0}, ".format(self.visit(i.context_expr))
-        with_statement = with_statement[:-2] + ") //unsupported\n{"
-        buf.append(with_statement)
-
-        for n in node.body:
-            buf.append(self.visit(n))
-
-            buf.append("}")
-
-        return "\n".join(buf)
-
     def visit_Await(self, node):
         return "await!({0})".format(self.visit(node.value))
 
@@ -559,9 +558,6 @@ class JuliaTranspiler(CLikeTranspiler):
             buf.append('println("{{:?}}",{0})'.format(value))
         return "\n".join(buf)
 
-    def visit_DictComp(self, node):
-        return "DictComp /*unimplemented()*/"
-
     def visit_GeneratorExp(self, node):
         elt = self.visit(node.elt)
         generator = node.generators[0]
