@@ -1,14 +1,24 @@
 import ast
-import functools
-import re
+
+from typing import List, Tuple
 
 from .clike import CLikeTranspiler
-from .declaration_extractor import DeclarationExtractor
 from .inference import get_inferred_kotlin_type
-from py2many.tracer import is_list, defined_before, is_class_or_module, is_self_arg
+from .plugins import (
+    ATTR_DISPATCH_TABLE,
+    CLASS_DISPATCH_TABLE,
+    FUNC_DISPATCH_TABLE,
+    MODULE_DISPATCH_TABLE,
+    DISPATCH_MAP,
+    SMALL_DISPATCH_MAP,
+    SMALL_USINGS_MAP,
+)
 
 from py2many.analysis import get_id, is_mutable, is_void_function
-from typing import Optional, List, Tuple
+from py2many.ast_helpers import create_ast_block
+from py2many.clike import class_for_typename
+from py2many.declaration_extractor import DeclarationExtractor
+from py2many.tracer import is_list, defined_before, is_class_or_module, is_self_arg
 
 
 class KotlinPrintRewriter(ast.NodeTransformer):
@@ -22,6 +32,7 @@ class KotlinPrintRewriter(ast.NodeTransformer):
 
     def visit_Call(self, node):
         fname = self.visit(node.func)
+
         if (
             get_id(fname) == "print"
             and len(node.args) == 1
@@ -31,21 +42,43 @@ class KotlinPrintRewriter(ast.NodeTransformer):
             )
         ):
             tmp = ast.Name(id=self._get_temp(), lineno=node.lineno)
-            ret = ast.If(
-                test=ast.Constant(value=True),
+            ret = create_ast_block(
                 body=[
                     ast.Assign(targets=[tmp], value=node.args[0], lineno=node.lineno),
                     node,
                 ],
-                orelse=[],
-                lineno=node.lineno,
+                at_node=node,
             )
             node.args[0] = tmp
+            ret.body_vars = [tmp]
             return ret
 
         return node
 
 
+class KotlinBitOpRewriter(ast.NodeTransformer):
+    BITOPS_DICT = {ast.BitAnd: "and", ast.BitOr: "or", ast.BitXor: "xor"}
+    BITOPS = tuple(BITOPS_DICT.keys())
+
+    def visit_BinOp(self, node):
+        left = node.left
+        right = node.right
+        op = node.op
+
+        if isinstance(op, self.BITOPS):
+            attr = self.BITOPS_DICT[type(op)]
+            ret = ast.Call(
+                func=ast.Attribute(value=left, attr=attr, ctx=ast.Load()),
+                args=[right],
+                keywords=[],
+            )
+            ret.lineno = node.lineno
+            ret.col_offset = node.col_offset
+            ast.fix_missing_locations(ret)
+            return ret
+        return node
+
+
 class KotlinTranspiler(CLikeTranspiler):
     NAME = "kotlin"
 
@@ -58,8 +91,14 @@ class KotlinTranspiler(CLikeTranspiler):
 
     def __init__(self):
         super().__init__()
-        self._default_type = "var"
+        self._default_type = ""
         self._container_type_map = self.CONTAINER_TYPE_MAP
+        self._dispatch_map = DISPATCH_MAP
+        self._small_dispatch_map = SMALL_DISPATCH_MAP
+        self._small_usings_map = SMALL_USINGS_MAP
+        self._func_dispatch_table = FUNC_DISPATCH_TABLE
+        self._attr_dispatch_table = ATTR_DISPATCH_TABLE
+        self._main_signature_arg_names = ["argv"]
 
     def usings(self):
         usings = sorted(list(set(self._usings)))
@@ -68,7 +107,7 @@ class KotlinTranspiler(CLikeTranspiler):
 
     def visit_FunctionDef(self, node):
         body = "\n".join([self.visit(n) for n in node.body])
-        typenames, args = self.visit(node.args)
+        typenames, args = self.visit_arguments(node.args)
 
         args_list = []
         if len(args) and hasattr(node, "self_type"):
@@ -78,9 +117,14 @@ class KotlinTranspiler(CLikeTranspiler):
 
         typedecls = []
         index = 0
+
+        is_python_main = getattr(node, "python_main", False)
+
         for i in range(len(args)):
             typename = typenames[i]
             arg = args[i]
+            if is_python_main and arg == "argc":
+                continue
             if typename == "T":
                 typename = "T{0}".format(index)
                 typedecls.append(typename)
@@ -91,8 +135,9 @@ class KotlinTranspiler(CLikeTranspiler):
         if not is_void_function(node):
             if node.returns:
                 typename = self._typename_from_annotation(node, attr="returns")
-                return_type = f": {typename}"
-            else:
+                if typename:
+                    return_type = f": {typename}"
+            if not return_type:
                 return_type = ": RT"
                 typedecls.append("RT")
 
@@ -141,6 +186,10 @@ class KotlinTranspiler(CLikeTranspiler):
 
         value_id = self.visit(node.value)
 
+        if value_id == "sys":
+            if attr == "argv":
+                return '(arrayOf("") + argv)'
+
         if not value_id:
             value_id = ""
 
@@ -152,69 +201,6 @@ class KotlinTranspiler(CLikeTranspiler):
 
         return f"{value_id}.{attr}"
 
-    def visit_range(self, node, vargs: List[str]) -> str:
-        if len(node.args) == 1:
-            return "(0..{}-1)".format(vargs[0])
-        elif len(node.args) == 2:
-            return "({}..{}-1)".format(vargs[0], vargs[1])
-        elif len(node.args) == 3:
-            return "({}..{}-1 step {})".format(vargs[0], vargs[1], vargs[2])
-
-        raise Exception(
-            "encountered range() call with unknown parameters: range({})".format(vargs)
-        )
-
-    def visit_print(self, node, vargs: List[str]) -> str:
-        def _format(arg):
-            if arg.isdigit():
-                return arg
-            if re.match(r"'.*'", arg) or re.match(r'".*"', arg):
-                return arg[1:-1]
-            else:
-                return f"${arg}"
-
-        vargs_str = " ".join([f"{_format(arg)}" for arg in vargs])
-        return f'println("{vargs_str}")'
-
-    def _dispatch(self, node, fname: str, vargs: List[str]) -> Optional[str]:
-        dispatch_map = {
-            "range": self.visit_range,
-            "xrange": self.visit_range,
-            "print": self.visit_print,
-        }
-
-        if fname in dispatch_map:
-            return dispatch_map[fname](node, vargs)
-
-        def visit_min_max(is_max: bool) -> str:
-            min_max = "max" if is_max else "min"
-            self._usings.add(f"kotlin.math.{min_max}")
-            self._typename_from_annotation(node.args[0])
-            if hasattr(node.args[0], "container_type"):
-                return f"maxOf({vargs[0]})"
-            else:
-                all_vargs = ", ".join(vargs)
-                return f"{min_max}({all_vargs})"
-
-        def visit_floor():
-            self._usings.add("kotlin.math.floor")
-            return f"floor({vargs[0]}).toInt()"
-
-        # small one liners are inlined here as lambdas
-        small_dispatch_map = {
-            "int": lambda: f"{vargs[0]}.toInt()",
-            "str": lambda: f"{vargs[0]}.toString()",
-            # TODO: strings use .length
-            "len": lambda: f"{vargs[0]}.size",
-            "max": functools.partial(visit_min_max, is_max=True),
-            "min": functools.partial(visit_min_max, is_min=True),
-            "floor": visit_floor,
-            "reversed": lambda: f"{vargs[0]}.reversed()",
-        }
-        if fname in small_dispatch_map:
-            return small_dispatch_map[fname]()
-        return None
-
     def visit_Call(self, node):
         fname = self.visit(node.func)
         vargs = []
@@ -316,6 +302,16 @@ class KotlinTranspiler(CLikeTranspiler):
         if ret is not None:
             return ret
 
+        decorators = [get_id(d) for d in node.decorator_list]
+        decorators = [
+            class_for_typename(t, None, self._imported_names) for t in decorators
+        ]
+        for d in decorators:
+            if d in CLASS_DISPATCH_TABLE:
+                ret = CLASS_DISPATCH_TABLE[d](self, node)
+                if ret is not None:
+                    return ret
+
         fields = []
         index = 0
         for declaration, typename in declarations.items():
@@ -380,21 +376,18 @@ class KotlinTranspiler(CLikeTranspiler):
             fields.append((member, var))
         return self._visit_enum(node, "Int", fields)
 
-    def visit_alias(self, node):
-        return "use {0}".format(node.name)
+    def _import(self, name: str) -> str:
+        return f"import {name};"
 
-    def visit_Import(self, node):
-        imports = [self.visit(n) for n in node.names]
-        return "\n".join(i for i in imports if i)
-
-    def visit_ImportFrom(self, node):
-        if node.module in self.IGNORED_MODULE_LIST:
-            return ""
-
-        names = [n.name for n in node.names]
-        names = ", ".join(names)
-        module_path = node.module.replace(".", "::")
-        return "use {0}::{{{1}}}".format(module_path, names)
+    def _import_from(self, module_name: str, names: List[str]) -> str:
+        if len(names) == 1:
+            # TODO: make this more generic so it works for len(names) > 1
+            name = names[0]
+            lookup = f"{module_name}.{name}"
+            if lookup in MODULE_DISPATCH_TABLE:
+                kt_module_name, kt_name = MODULE_DISPATCH_TABLE[lookup]
+                return f"from {kt_module_name} import {kt_name}"
+        return "\n".join([f"import {module_name}.{name}" for name in names])
 
     def visit_List(self, node):
         elements = [self.visit(e) for e in node.elts]
@@ -421,7 +414,14 @@ class KotlinTranspiler(CLikeTranspiler):
             if value == "Tuple":
                 return "({0})".format(index)
             return "{0}<{1}>".format(value, index)
-        return "{0}[{1}]".format(value, index)
+        self._generic_typename_from_annotation(node.value)
+        container_type = hasattr(node.value, "annotation") and getattr(
+            node.value.annotation, "generic_container_type", None
+        )
+        is_container_dict = (
+            container_type and len(container_type) > 0 and container_type[0] == "Dict"
+        )
+        return f"{value}[{index}]!!" if is_container_dict else f"{value}[{index}]"
 
     def visit_Index(self, node):
         return self.visit(node.value)
@@ -443,21 +443,15 @@ class KotlinTranspiler(CLikeTranspiler):
             return elts
         return "({0})".format(elts)
 
-    def visit_unsupported_body(self, name, body):
-        buf = ["let {0} = {{ //unsupported".format(name)]
-        buf += [self.visit(n) for n in body]
-        buf.append("};")
-        return buf
-
     def visit_Try(self, node, finallybody=None):
-        buf = self.visit_unsupported_body("try_dummy", node.body)
+        buf = self.visit_unsupported_body(node, "try_dummy", node.body)
 
         for handler in node.handlers:
             buf += self.visit(handler)
         # buf.append("\n".join(excepts));
 
         if finallybody:
-            buf += self.visit_unsupported_body("finally_dummy", finallybody)
+            buf += self.visit_unsupported_body(node, "finally_dummy", finallybody)
 
         return "\n".join(buf)
 
@@ -466,7 +460,7 @@ class KotlinTranspiler(CLikeTranspiler):
         if node.type:
             exception_type = self.visit(node.type)
         name = "except!({0})".format(exception_type)
-        body = self.visit_unsupported_body(name, node.body)
+        body = self.visit_unsupported_body(node, name, node.body)
         return body
 
     def visit_Assert(self, node):
@@ -504,7 +498,9 @@ class KotlinTranspiler(CLikeTranspiler):
             value = self.visit(node.value)
             return f"{target} = {value}"
 
-        definition = node.scopes.find(target.id)
+        definition = node.scopes.parent_scopes.find(get_id(target))
+        if definition is None:
+            definition = node.scopes.find(get_id(target))
         if isinstance(target, ast.Name) and defined_before(definition, node):
             target = self.visit(target)
             value = self.visit(node.value)
@@ -531,27 +527,6 @@ class KotlinTranspiler(CLikeTranspiler):
             return f"throw Exception({exc})"
         return "throw Exception()"
 
-    def visit_With(self, node):
-        buf = []
-
-        with_statement = "// with!("
-        for i in node.items:
-            if i.optional_vars:
-                with_statement += "{0} as {1}, ".format(
-                    self.visit(i.context_expr), self.visit(i.optional_vars)
-                )
-            else:
-                with_statement += "{0}, ".format(self.visit(i.context_expr))
-        with_statement = with_statement[:-2] + ") //unsupported\n{"
-        buf.append(with_statement)
-
-        for n in node.body:
-            buf.append(self.visit(n))
-
-            buf.append("}")
-
-        return "\n".join(buf)
-
     def visit_Await(self, node):
         expr = self.visit(node.value)
         return f"{expr}.await()"
@@ -567,9 +542,6 @@ class KotlinTranspiler(CLikeTranspiler):
         vargs_str = " ".join([f"${arg}" for arg in node.values])
         return f'println("{vargs_str}")'
 
-    def visit_DictComp(self, node):
-        return "DictComp /*unimplemented()*/"
-
     def visit_GeneratorExp(self, node):
         return "GeneratorExp /*unimplemented()*/"
 
