@@ -2,11 +2,24 @@ import ast
 
 from .clike import CLikeTranspiler
 from .inference import get_inferred_nim_type
-from py2many.declaration_extractor import DeclarationExtractor
-from py2many.tracer import is_list, defined_before
+from .plugins import (
+    ATTR_DISPATCH_TABLE,
+    CLASS_DISPATCH_TABLE,
+    FUNC_DISPATCH_TABLE,
+    FUNC_USINGS_MAP,
+    MODULE_DISPATCH_TABLE,
+    DISPATCH_MAP,
+    SMALL_DISPATCH_MAP,
+    SMALL_USINGS_MAP,
+)
 
 from py2many.analysis import get_id, is_mutable, is_void_function
-from typing import Optional, List
+from py2many.clike import class_for_typename
+from py2many.declaration_extractor import DeclarationExtractor
+from py2many.exceptions import AstClassUsedBeforeDeclaration
+from py2many.tracer import is_list, defined_before
+
+from typing import List
 
 
 class NimNoneCompareRewriter(ast.NodeTransformer):
@@ -34,14 +47,20 @@ class NimTranspiler(CLikeTranspiler):
         "Optional": "Option",
     }
 
-    ALLOW_MODULE_LIST = ["math"]
-
     def __init__(self, indent=2):
         super().__init__()
         self._headers = set([])
         self._indent = " " * indent
         self._default_type = "var"
         self._container_type_map = self.CONTAINER_TYPE_MAP
+        if "math" in self._ignored_module_set:
+            self._ignored_module_set.remove("math")
+        self._dispatch_map = DISPATCH_MAP
+        self._small_dispatch_map = SMALL_DISPATCH_MAP
+        self._small_usings_map = SMALL_USINGS_MAP
+        self._func_dispatch_table = FUNC_DISPATCH_TABLE
+        self._attr_dispatch_table = ATTR_DISPATCH_TABLE
+        self._func_usings_map = FUNC_USINGS_MAP
 
     def indent(self, code, level=1):
         return self._indent * level + code
@@ -61,6 +80,8 @@ class NimTranspiler(CLikeTranspiler):
         body = "\n".join([self.indent(self.visit(n)) for n in node.body])
         typenames, args = self.visit(node.args)
 
+        is_python_main = getattr(node, "python_main", False)
+
         args_list = []
         if len(args) and hasattr(node, "self_type"):
             typenames[0] = node.self_type
@@ -68,6 +89,7 @@ class NimTranspiler(CLikeTranspiler):
         for i in range(len(args)):
             typename = typenames[i]
             arg = args[i]
+
             args_list.append(f"{arg}: {typename}".format(arg, typename))
 
         return_type = ""
@@ -81,7 +103,7 @@ class NimTranspiler(CLikeTranspiler):
         args = ", ".join(args_list)
         funcdef = f"proc {node.name}({args}){return_type} ="
         maybe_main = ""
-        if getattr(node, "python_main", False):
+        if is_python_main:
             maybe_main = "\nmain()"
         return f"{funcdef}\n{body}\n{maybe_main}"
 
@@ -101,10 +123,6 @@ class NimTranspiler(CLikeTranspiler):
             return f"return {ret}"
         return "return"
 
-        if node.value:
-            return "return {0}".format(self.visit(node.value))
-        return "return"
-
     def visit_arg(self, node):
         id = get_id(node)
         if id == "self":
@@ -136,59 +154,29 @@ class NimTranspiler(CLikeTranspiler):
 
         value_id = self.visit(node.value)
 
+        if value_id == "sys":
+            if attr == "argv":
+                self._usings.add("os")
+                return "(@[getAppFilename()] & commandLineParams())"
+
         if is_list(node.value):
             if node.attr == "append":
                 attr = "add"
         if not value_id:
             value_id = ""
 
-        return value_id + "." + attr
-
-    def visit_range(self, node, vargs: List[str]) -> str:
-        if len(node.args) == 1:
-            return f"(0..{vargs[0]} - 1)"
-        elif len(node.args) == 2:
-            return f"({vargs[0]}..{vargs[1]} - 1)"
-        elif len(node.args) == 3:
-            return f"countup({vargs[0]}, {vargs[1]} - 1, {vargs[2]})"
-
-        raise Exception(
-            "encountered range() call with unknown parameters: range({})".format(vargs)
-        )
-
-    def visit_print(self, node, vargs: List[str]) -> str:
-        args = []
-        for n in vargs:
-            args.append(n)
-            args.append('" "')
-        args = ", ".join(args[:-1])
-        return f"echo {args}"
-
-    def _dispatch(self, node, fname: str, vargs: List[str]) -> Optional[str]:
-        dispatch_map = {
-            "range": self.visit_range,
-            "xrange": self.visit_range,
-            "print": self.visit_print,
-        }
-
-        if fname in dispatch_map:
-            return dispatch_map[fname](node, vargs)
-
-        # small one liners are inlined here as lambdas
-        small_dispatch_map = {
-            "str": lambda: f"$({vargs[0]})",
-            "floor": lambda: f"int(floor({vargs[0]}))",
-        }
-        if fname in small_dispatch_map:
-            return small_dispatch_map[fname]()
-        return None
+        ret = f"{value_id}.{attr}"
+        if ret in self._attr_dispatch_table:
+            ret = self._attr_dispatch_table[ret]
+            return ret(self, node, value_id, attr)
+        return ret
 
     def _visit_object_literal(self, node, fname: str, fndef: ast.ClassDef):
         vargs = []  # visited args
         if not hasattr(fndef, "declarations"):
-            raise Exception("Missing declarations")
+            raise AstClassUsedBeforeDeclaration(fndef, node)
         if node.args:
-            for arg, decl in zip(node.args, fndef.declaration.keys()):
+            for arg, decl in zip(node.args, fndef.declarations.keys()):
                 arg = self.visit(arg)
                 vargs += [f"{decl}: {arg}"]
         if node.keywords:
@@ -313,6 +301,16 @@ class NimTranspiler(CLikeTranspiler):
         if ret is not None:
             return ret
 
+        decorators = [get_id(d) for d in node.decorator_list]
+        decorators = [
+            class_for_typename(t, None, self._imported_names) for t in decorators
+        ]
+        for d in decorators:
+            if d in CLASS_DISPATCH_TABLE:
+                ret = CLASS_DISPATCH_TABLE[d](self, node)
+                if ret is not None:
+                    return ret
+
         fields = []
         index = 0
         for declaration, typename in declarations.items():
@@ -374,24 +372,19 @@ class NimTranspiler(CLikeTranspiler):
         fields = "\n".join([self.indent(f) for f in fields])
         return f"type {node.name} = enum\n{fields}\n\n"
 
-    def visit_alias(self, node):
-        return "use {0}".format(node.name)
+    def _import(self, name: str) -> str:
+        return f"import {name}"
 
-    def visit_Import(self, node):
-        imports = [self.visit(n) for n in node.names]
-        return "\n".join(i for i in imports if i)
-
-    def visit_ImportFrom(self, node):
-        if (
-            node.module not in self.ALLOW_MODULE_LIST
-            and node.module in self.IGNORED_MODULE_LIST
-        ):
-            return ""
-
-        names = [n.name for n in node.names]
+    def _import_from(self, module_name: str, names: List[str]) -> str:
         names = ", ".join(names)
-        module_path = node.module.replace(".", "::")
-        return f"from {module_path} import {names}"
+        if len(names) == 1:
+            # TODO: make this more generic so it works for len(names) > 1
+            name = names[0]
+            lookup = f"{module_name}.{name}"
+            if lookup in MODULE_DISPATCH_TABLE:
+                nim_module_name, nim_name = MODULE_DISPATCH_TABLE[lookup]
+                return f"from {nim_module_name} import {nim_name}"
+        return f"from {module_name} import {names}"
 
     def visit_List(self, node):
         elements = [self.visit(e) for e in node.elts]
@@ -442,21 +435,15 @@ class NimTranspiler(CLikeTranspiler):
             return elts
         return "({0})".format(elts)
 
-    def visit_unsupported_body(self, name, body):
-        buf = ["let {0} = {{ //unsupported".format(name)]
-        buf += [self.visit(n) for n in body]
-        buf.append("};")
-        return buf
-
     def visit_Try(self, node, finallybody=None):
-        buf = self.visit_unsupported_body("try_dummy", node.body)
+        buf = self.visit_unsupported_body(node, "try_dummy", node.body)
 
         for handler in node.handlers:
             buf += self.visit(handler)
         # buf.append("\n".join(excepts));
 
         if finallybody:
-            buf += self.visit_unsupported_body("finally_dummy", finallybody)
+            buf += self.visit_unsupported_body(node, "finally_dummy", finallybody)
 
         return "\n".join(buf)
 
@@ -465,7 +452,7 @@ class NimTranspiler(CLikeTranspiler):
         if node.type:
             exception_type = self.visit(node.type)
         name = "except!({0})".format(exception_type)
-        body = self.visit_unsupported_body(name, node.body)
+        body = self.visit_unsupported_body(node, name, node.body)
         return body
 
     def visit_Assert(self, node):
@@ -507,17 +494,13 @@ class NimTranspiler(CLikeTranspiler):
             value = self.visit(node.value)
             return f"{target} = {value}"
 
-        definition = node.scopes.find(target.id)
+        definition = node.scopes.parent_scopes.find(get_id(target))
+        if definition is None:
+            definition = node.scopes.find(get_id(target))
         if isinstance(target, ast.Name) and defined_before(definition, node):
             target = self.visit(target)
             value = self.visit(node.value)
             return f"{target} = {value}"
-        elif isinstance(node.value, ast.List):
-            elements = [self.visit(e) for e in node.value.elts]
-            elements = ", ".join(elements)
-            target = self.visit(target)
-
-            return f"{kw} {target} = @[{elements}]"
         else:
             target = self.visit(target)
             value = self.visit(node.value)
@@ -535,27 +518,6 @@ class NimTranspiler(CLikeTranspiler):
         # specifying the exception.
         return "raise!(); //unsupported"
 
-    def visit_With(self, node):
-        buf = []
-
-        with_statement = "// with!("
-        for i in node.items:
-            if i.optional_vars:
-                with_statement += "{0} as {1}, ".format(
-                    self.visit(i.context_expr), self.visit(i.optional_vars)
-                )
-            else:
-                with_statement += "{0}, ".format(self.visit(i.context_expr))
-        with_statement = with_statement[:-2] + ") //unsupported\n{"
-        buf.append(with_statement)
-
-        for n in node.body:
-            buf.append(self.visit(n))
-
-            buf.append("}")
-
-        return "\n".join(buf)
-
     def visit_Await(self, node):
         return "await!({0})".format(self.visit(node.value))
 
@@ -563,8 +525,10 @@ class NimTranspiler(CLikeTranspiler):
         return "#[async]\n{0}".format(self.visit_FunctionDef(node))
 
     def visit_Yield(self, node):
-        value = self.visit(node.value)
-        return f"yield {value}"
+        if node.value is not None:
+            value = self.visit(node.value)
+            return f"yield {value}"
+        return "yield nil"
 
     def visit_Print(self, node):
         buf = []
@@ -573,9 +537,6 @@ class NimTranspiler(CLikeTranspiler):
             buf.append('println("{{:?}}",{0})'.format(value))
         return "\n".join(buf)
 
-    def visit_DictComp(self, node):
-        return "DictComp /*unimplemented()*/"
-
     def visit_GeneratorExp(self, node):
         elt = self.visit(node.elt)
         generator = node.generators[0]
