@@ -1,16 +1,35 @@
 import ast
-import functools
-from textwrap import dedent
+from py2many.rewriters import camel_case
+import textwrap
 
 from .clike import CLikeTranspiler
-from .declaration_extractor import DeclarationExtractor
-from .inference import get_inferred_rust_type
+from .inference import get_inferred_rust_type, map_type
+from .plugins import (
+    ATTR_DISPATCH_TABLE,
+    CLASS_DISPATCH_TABLE,
+    FUNC_DISPATCH_TABLE,
+    FUNC_USINGS_MAP,
+    MODULE_DISPATCH_TABLE,
+    DISPATCH_MAP,
+    SMALL_DISPATCH_MAP,
+    SMALL_USINGS_MAP,
+)
 
-from py2many.analysis import get_id, is_global, is_mutable, is_void_function
+from py2many.analysis import (
+    FunctionTransformer,
+    get_id,
+    is_global,
+    is_mutable,
+    is_void_function,
+)
+from py2many.clike import class_for_typename
+from py2many.declaration_extractor import DeclarationExtractor
+from py2many.exceptions import AstClassUsedBeforeDeclaration
 from py2many.inference import is_reference
 from py2many.tracer import is_list, defined_before, is_class_or_module
 
-from typing import List, Optional
+from pathlib import Path
+from typing import List, Tuple, Union
 
 
 class RustLoopIndexRewriter(ast.NodeTransformer):
@@ -64,27 +83,56 @@ class RustTranspiler(CLikeTranspiler):
         "Dict": "HashMap",
         "Set": "HashSet",
         "Optional": "Option",
+        "Result": "Result",
     }
 
-    def __init__(self):
+    def __init__(self, extension: bool = False, no_prologue: bool = False):
         super().__init__()
         self._container_type_map = self.CONTAINER_TYPE_MAP
         self._default_type = "_"
+        self._extension = extension
+        self._rust_ignored_module_set = {"argparse_dataclass"}
+        self._no_prologue = no_prologue
+        self._dispatch_map = DISPATCH_MAP
+        self._small_dispatch_map = SMALL_DISPATCH_MAP
+        self._small_usings_map = SMALL_USINGS_MAP
+        self._func_dispatch_table = FUNC_DISPATCH_TABLE
+        self._func_usings_map = FUNC_USINGS_MAP
+        self._attr_dispatch_table = ATTR_DISPATCH_TABLE
+        self._allows = set()
 
     def usings(self):
+        if self._extension:
+            self._usings.add("pyo3::prelude::*")
+            self._usings.add("pyo3::wrap_pyfunction")
         usings = sorted(list(set(self._usings)))
         deps = sorted(
             set(mod.split("::")[0] for mod in usings if not mod.startswith("std:"))
         )
         externs = [f"extern crate {dep};" for dep in deps]
-        deps = "\n//! ".join([f'{dep} = "*"' for dep in deps])
+        deps_str = "\n//! ".join([f'{dep} = "*"' for dep in deps])
         externs = "\n".join(externs)
         uses = "\n".join(
-            f"use {mod};" for mod in usings if mod not in ("strum", "lazy_static")
+            f"use {mod};"
+            for mod in usings
+            if mod not in ("strum", "lazy_static") and mod not in deps
         )
-        lint_ignores = dedent(
-            """
+        # This should not contain lints which mask possibly erroneous semantics, e.g. float_cmp
+        # Those, and lints triggered by explicitly bad test logic, belong in
+        # .github/workflows/clippy.yml
+        lint_ignores = (
+            textwrap.dedent(
+                """
+        #![allow(clippy::collapsible_else_if)]
+        #![allow(clippy::double_parens)]  // https://github.com/adsharma/py2many/issues/17
+        #![allow(clippy::map_identity)]
+        #![allow(clippy::needless_return)]
+        #![allow(clippy::print_literal)]
+        #![allow(clippy::ptr_arg)]
+        #![allow(clippy::redundant_static_lifetimes)]  // https://github.com/adsharma/py2many/issues/266
+        #![allow(clippy::unnecessary_cast)]
         #![allow(clippy::upper_case_acronyms)]
+        #![allow(clippy::useless_vec)]
         #![allow(non_camel_case_types)]
         #![allow(non_snake_case)]
         #![allow(non_upper_case_globals)]
@@ -92,22 +140,59 @@ class RustTranspiler(CLikeTranspiler):
         #![allow(unused_mut)]
         #![allow(unused_parens)]
         """
+            )
+            if not self._no_prologue
+            else ""
         )
-        cargo_toml = f"""
+        lint_ignores += "\n".join(f"#![allow({allow})]" for allow in self._allows)
+        cargo_toml = (
+            f"""\
         //! ```cargo
         //! [package]
         //! edition = "2018"
         //! [dependencies]
-        //! {deps}
+        //! {deps_str}
         //! ```
         """
-        return f"{cargo_toml}\n{lint_ignores}\n{externs}\n{uses}\n"
+            if not self._no_prologue
+            else ""
+        )
+        return f"{cargo_toml}\n{lint_ignores}\n\n{externs}\n{uses}\n"
 
     def features(self):
         if self._features:
             features = ", ".join(sorted(list(set(self._features))))
             return f"#![feature({features})]"
 
+    def extension_module(self, tree) -> str:
+        if self._extension:
+            funcs = []
+            tx = FunctionTransformer()
+            tx.visit(tree)
+            for f in tree.defined_functions:
+                fname = f.name
+                if fname.startswith("_"):
+                    continue
+                funcs.append(f"m.add_function(wrap_pyfunction!({fname}, m)?)?;")
+            funcs_str = "\n".join(funcs)
+            module_name = Path(tree.__file__).stem
+            return textwrap.dedent(
+                f"""\
+            #[pymodule]
+            fn {module_name}(_py: Python, m: &PyModule) -> PyResult<()> {{
+                {funcs_str}
+
+                Ok(())
+            }}
+            """
+            )
+        return ""
+
+    def visit_Expr(self, node):
+        if hasattr(node, "unused"):
+            self._allows.add("clippy::no_effect")
+        return super().visit_Expr(node)
+
     def visit_FunctionDef(self, node, async_prefix=""):
         body = "\n".join([self.visit(n) for n in node.body])
         typenames, args = self.visit(node.args)
@@ -118,25 +203,37 @@ class RustTranspiler(CLikeTranspiler):
             del args[0]
             args_list.append("&self")
 
+        is_python_main = getattr(node, "python_main", False)
+        if is_python_main:
+            self._usings.add("anyhow::Result")
+
         typedecls = []
         index = 0
         for i in range(len(args)):
             typename = typenames[i]
             arg = args[i]
+
             if typename == "T":
                 typename = "T{0}".format(index)
                 typedecls.append(typename)
                 index += 1
             args_list.append("{0}: {1}".format(arg, typename))
 
-        return_type = ""
-        if not is_void_function(node):
-            if node.returns:
-                typename = self._typename_from_annotation(node, attr="returns")
-                if getattr(node.returns, "rust_needs_reference", False):
-                    typename = f"&{typename}"
+        return_type = "" if not is_python_main else "-> Result<()>"
+        if node.returns:
+            typename = self._typename_from_annotation(node, attr="returns")
+            if getattr(node.returns, "rust_needs_reference", False):
+                typename = f"&{typename}"
+            if getattr(node, "rust_pyresult_type", False):
+                if node.no_return:
+                    typename = None
+                else:
+                    typename = self._generic_typename_from_type_node(node.returns)
+                typename = map_type(typename, extension=True, return_type=True)
+            if typename != "_":
                 return_type = f"-> {typename}"
-            else:
+        else:
+            if not is_void_function(node):
                 return_type = "-> RT"
                 typedecls.append("RT")
 
@@ -144,11 +241,13 @@ class RustTranspiler(CLikeTranspiler):
         if len(typedecls) > 0:
             template = "<{0}>".format(", ".join(typedecls))
 
+        extension = "#[pyfunction]\n" if self.extension else ""
         args_list = ", ".join(args_list)
-        funcdef = (
-            f"pub {async_prefix}fn {node.name}{template}({args_list}) {return_type}"
-        )
-        return funcdef + " {\n" + body + "\n}\n"
+        funcdef = f"{extension}pub {async_prefix}fn {node.name}{template}({args_list}) {return_type}"
+        return_success = (
+            "Ok(())" if is_python_main else ""
+        )  # TODO: generalize this to functions that return Result<T, E>
+        return f"{funcdef} {{\n{body}\n {return_success}}}\n"
 
     def visit_arg(self, node):
         id = get_id(node)
@@ -156,7 +255,11 @@ class RustTranspiler(CLikeTranspiler):
             return (None, "self")
         typename = "T"
         if node.annotation:
-            typename = self._typename_from_annotation(node)
+            if not self._extension:
+                typename = self._typename_from_annotation(node)
+            else:
+                typename = self._generic_typename_from_annotation(node)
+                typename = map_type(typename, extension=True)
             mut = "mut " if is_mutable(node.scopes, id) else ""
             # TODO: Should we make this if not primitive instead of checking
             # for container types? That way we cover user defined structs too.
@@ -166,14 +269,20 @@ class RustTranspiler(CLikeTranspiler):
         return (typename, id)
 
     def visit_Return(self, node):
+        fndef = None
+        for scope in node.scopes:
+            if isinstance(scope, ast.FunctionDef):
+                fndef = scope
+                break
         if node.value:
             ret = self.visit(node.value)
-            fndef = None
-            for scope in node.scopes:
-                if isinstance(scope, ast.FunctionDef):
-                    fndef = scope
-                    break
             if fndef:
+                if getattr(fndef, "rust_pyresult_type", False):
+                    # TODO: Design a more robust solution for this
+                    # For now, PyResult and references don't mix
+                    if ret.startswith("&"):
+                        ret = ret[1:]
+                    ret = f"Ok({ret})"
                 return_type = self._typename_from_annotation(fndef, attr="returns")
                 value_type = get_inferred_rust_type(node.value)
                 if is_reference(node.value) and not getattr(
@@ -184,6 +293,9 @@ class RustTranspiler(CLikeTranspiler):
                 if return_type != value_type and value_type is not None:
                     return f"return {ret} as {return_type};"
             return f"return {ret};"
+        if fndef:
+            if getattr(fndef, "rust_pyresult_type", False):
+                return "return Ok(())"
         return "return;"
 
     def visit_Lambda(self, node):
@@ -194,9 +306,16 @@ class RustTranspiler(CLikeTranspiler):
 
     def visit_Attribute(self, node):
         attr = node.attr
+        if attr in self._keywords:
+            attr = attr + "_"
 
         value_id = self.visit(node.value)
 
+        if value_id == "sys":
+            if attr == "argv":
+                self._usings.add("std::env")
+                return "env::args().map(|s| &*Box::leak(s.into_boxed_str())).collect()"
+
         if is_list(node.value):
             if node.attr == "append":
                 attr = "push"
@@ -206,93 +325,39 @@ class RustTranspiler(CLikeTranspiler):
         if is_class_or_module(value_id, node.scopes):
             return "{0}::{1}".format(value_id, attr)
 
-        return value_id + "." + attr
+        ret = f"{value_id}.{attr}"
+        if ret in self._attr_dispatch_table:
+            ret = self._attr_dispatch_table[ret]
+            return ret(self, node, value_id, attr)
+        return ret
 
-    def _dispatch(self, node, fname: str, vargs: List[str]) -> Optional[str]:
-        def visit_range(node, vargs: List[str]) -> str:
-            if len(node.args) == 1:
-                return "(0..{})".format(vargs[0])
-            elif len(node.args) == 2:
-                return "({}..{})".format(vargs[0], vargs[1])
-            elif len(node.args) == 3:
-                return "({}..{}).step_by({})".format(vargs[0], vargs[1], vargs[2])
+    def _func_for_lookup(self, fname) -> Union[str, object]:
+        fname_for_lookup = fname.replace("::", ".")
+        return super()._func_for_lookup(fname_for_lookup)
 
-            raise Exception(
-                "encountered range() call with unknown parameters: range({})".format(
-                    vargs
-                )
-            )
-
-        def visit_print(node, vargs: List[str]) -> str:
-            placeholders = []
-            for n in node.args:
-                placeholders.append("{}")
-            return 'println!("{0}",{1});'.format(
-                " ".join(placeholders), ", ".join(vargs)
-            )
-
-        dispatch_map = {
-            "range": visit_range,
-            "xrange": visit_range,
-            "print": visit_print,
-        }
-
-        if fname in dispatch_map:
-            return dispatch_map[fname](node, vargs)
-
-        def visit_min_max(is_max: bool) -> str:
-            self._usings.add("std::cmp")
-            min_max = "max" if is_max else "min"
-            self._typename_from_annotation(node.args[0])
-            if hasattr(node.args[0], "container_type"):
-                return f"{vargs[0]}.iter().{min_max}().unwrap()"
-            else:
-                all_vargs = ", ".join(vargs)
-                return f"cmp::{min_max}({all_vargs})"
-
-        def visit_cast(cast_to: str) -> str:
-            if "()" in vargs[0]:
-                return f"{vargs[0]} as {cast_to}"
-            else:
-                return f"{cast_to}::from({vargs[0]})"
-
-        def visit_asyncio_run() -> str:
-            self._usings.add("futures::executor::block_on")
-            return f"block_on({vargs[0]})"
-
-        # small one liners are inlined here as lambdas
-        small_dispatch_map = {
-            "str": lambda: f"&{vargs[0]}.to_string()",
-            "len": lambda: f"{vargs[0]}.len()",
-            "enumerate": lambda: f"{vargs[0]}.iter().enumerate()",
-            "sum": lambda: f"{vargs[0]}.iter().sum()",
-            "int": functools.partial(visit_cast, cast_to="i32"),
-            "float": functools.partial(visit_cast, cast_to="f64"),
-            "max": functools.partial(visit_min_max, is_max=True),
-            "min": functools.partial(visit_min_max, is_min=True),
-            # as usize below is a hack to pass comb_sort.rs. Need a better solution
-            "floor": lambda: f"{vargs[0]}.floor() as usize",
-            "reversed": lambda: f"{vargs[0]}.iter().rev()",
-            "map": lambda: f"{vargs[1]}.iter().map({vargs[0]})",
-            "filter": lambda: f"{vargs[1]}.into_iter().filter({vargs[0]})",
-            "list": lambda: f"{vargs[0]}.collect::<Vec<_>>()",
-            "asyncio.run": visit_asyncio_run,
-        }
-        if fname in small_dispatch_map:
-            return small_dispatch_map[fname]()
-        return None
+    def _func_name_split(self, fname: str) -> Tuple[str, str]:
+        # string based fallback
+        splits = fname.rsplit("::", maxsplit=1)
+        if len(splits) == 2:
+            return tuple(splits)
+        else:
+            return ("", splits[0])
 
     def _visit_struct_literal(self, node, fname: str, fndef: ast.ClassDef):
         vargs = []  # visited args
         if not hasattr(fndef, "declarations"):
-            raise Exception("Missing declarations")
+            raise AstClassUsedBeforeDeclaration(fndef, node)
         if node.args:
-            for arg, decl in zip(node.args, fndef.declaration.keys()):
+            for arg, decl in zip(node.args, fndef.declarations.keys()):
                 arg = self.visit(arg)
+                if decl in self._keywords:
+                    decl += "_"
                 vargs += [f"{decl}: {arg}"]
         if node.keywords:
             for kw in node.keywords:
                 value = self.visit(kw.value)
+                if kw.arg in self._keywords:
+                    kw.arg += "_"
                 vargs += [f"{kw.arg}: {value}"]
         args = ", ".join(vargs)
         return f"{fname}{{{args}}}"
@@ -311,8 +376,13 @@ class RustTranspiler(CLikeTranspiler):
             vargs += [self.visit(kw.value) for kw in node.keywords]
 
         ret = self._dispatch(node, fname, vargs)
+        node_result_type = getattr(node, "result_type", False)
+        node_func_result_type = getattr(node.func, "result_type", False)
         if ret is not None:
-            return ret
+            if ret.startswith("std::process::exit("):
+                node_result_type = False
+            unwrap = "?" if node_result_type or node_func_result_type else ""
+            return f"{ret}{unwrap}"
 
         # Check if some args need to be passed by reference
         ref_args = []
@@ -326,7 +396,8 @@ class RustTranspiler(CLikeTranspiler):
             ref_args = vargs
 
         args = ", ".join(ref_args)
-        return f"{fname}({args})"
+        unwrap = "?" if node_result_type or node_func_result_type else ""
+        return f"{fname}({args}){unwrap}"
 
     def visit_For(self, node):
         target = self.visit(node.target)
@@ -341,20 +412,29 @@ class RustTranspiler(CLikeTranspiler):
         return "" + super().visit_Str(node) + ""
 
     def visit_Bytes(self, node):
-        bytes_str = "{0}".format(node.s)
-        return bytes_str.replace("'", '"')  # replace single quote with double quote
+        bytes_str = node.s
+        bytes_str = bytes_str.replace(b'"', b'\\"')
+        return 'b"' + bytes_str.decode("ascii", "backslashreplace") + '"'
 
     def visit_Compare(self, node):
         left = self.visit(node.left)
         right = self.visit(node.comparators[0])
+
+        if hasattr(node.comparators[0], "annotation"):
+            self._generic_typename_from_annotation(node.comparators[0])
+            value_type = getattr(
+                node.comparators[0].annotation, "generic_container_type", None
+            )
+            if value_type and value_type[0] == "Dict":
+                right += ".keys()"
+
+        if not right.endswith(".keys()") and not right.endswith(".values()"):
+            right += ".iter()"
+
         if isinstance(node.ops[0], ast.In):
-            return "{0}.iter().any(|&x| x == {1})".format(
-                right, left
-            )  # is it too much?
+            return "{0}.any(|&x| x == {1})".format(right, left)  # is it too much?
         elif isinstance(node.ops[0], ast.NotIn):
-            return "{0}.iter().all(|&x| x != {1})".format(
-                right, left
-            )  # is it even more?
+            return "{0}.all(|&x| x != {1})".format(right, left)  # is it even more?
 
         return super().visit_Compare(node)
 
@@ -405,6 +485,10 @@ class RustTranspiler(CLikeTranspiler):
         return ret
 
     def visit_While(self, node):
+        test = self.visit(node.test)
+        if test == "true":
+            body = [self.visit(n) for n in node.body]
+            return "loop {{\n{}\n}}\n".format("\n".join(body))
         return super().visit_While(node, use_parens=False)
 
     def visit_UnaryOp(self, node):
@@ -423,35 +507,89 @@ class RustTranspiler(CLikeTranspiler):
             and isinstance(node.op, ast.Mult)
             and isinstance(node.right, ast.Num)
         ):
-            return "std::vector ({0},{1})".format(
-                self.visit(node.right), self.visit(node.left.elts[0])
-            )
+            elt, n = self.visit(node.left.elts[0]), self.visit(node.right)
+            return f"vec![{elt};{n}]"
         else:
             return super().visit_BinOp(node)
 
+    def visit_sealed_class(self, node):
+        variants = []
+        accessors = []
+        tag_checkers = []
+        camel_node_name = camel_case(node.name)
+        for member, var in node.class_assignments.items():
+            member_id = get_id(member)
+            camel_member_id = camel_case(member_id)
+            lower_member_id = member_id.lower()
+            typename = node.declarations_with_defaults.get(member_id, None)
+            if typename == None:
+                variants.append(f"{member_id},")
+            else:
+                typename, _ = typename
+                variants.append(f"{member_id}({typename}),")
+            tag_check = textwrap.dedent(
+                f"""
+                    fn is_{lower_member_id}(&self) -> bool {{
+                        matches!(*self, {camel_node_name}::{camel_member_id}(_))
+                    }}"""
+            )
+            tag_checkers.append(tag_check)
+            accessor = textwrap.dedent(
+                f"""
+                    fn {lower_member_id}(&self) -> Option<&{typename}>{{
+                        if let {camel_node_name}::{camel_member_id}(val) = self {{
+                            Some(val)
+                        }} else {{
+                            None
+                        }}
+                    }}"""
+            )
+            accessors.append(accessor)
+        body_str = "\n".join(variants)
+        impl_str = "\n".join(accessors) + "\n" + "\n".join(tag_checkers)
+        return f"enum {camel_node_name} {{ {body_str} }}\n\n impl {camel_node_name} {{ {impl_str} }}\n\n"
+
     def visit_ClassDef(self, node):
         extractor = DeclarationExtractor(RustTranspiler())
         extractor.visit(node)
         node.declarations = declarations = extractor.get_declarations()
+        node.declarations_with_defaults = extractor.get_declarations_with_defaults()
         node.class_assignments = extractor.class_assignments
         ret = super().visit_ClassDef(node)
         if ret is not None:
             return ret
 
+        decorators = [get_id(d) for d in node.decorator_list]
+        if "sealed" in decorators:
+            # TODO: handle cases where sealed is stacked with other decorators
+            return self.visit_sealed_class(node)
+
+        decorators = [
+            class_for_typename(t, None, self._imported_names) for t in decorators
+        ]
+        for d in decorators:
+            if d in CLASS_DISPATCH_TABLE:
+                ret = CLASS_DISPATCH_TABLE[d](self, node)
+                if ret is not None:
+                    return ret
+
         fields = []
         index = 0
         for declaration, typename in declarations.items():
             if typename == None:
                 typename = "ST{0}".format(index)
                 index += 1
-            fields.append("pub {0}: {1},".format(declaration, typename))
+            fields.append(f"pub {declaration}: {typename},")
 
+        extension = "#[pyclass]\n" if self.extension else ""
         struct_def = "pub struct {0} {{\n{1}\n}}\n\n".format(
             node.name, "\n".join(fields)
         )
-        impl_def = "impl {0} {{\n".format(node.name)
+        impl_extension = "#[pymethods]\n" if self.extension else ""
+        impl_def = f"{impl_extension}impl {node.name} {{\n"
         buf = [self.visit(b) for b in node.body]
-        return "{0}{1}{2} \n}}".format(struct_def, impl_def, "\n".join(buf))
+        buf_str = "\n".join(buf)
+        return f"{extension}{struct_def}{impl_def}{buf_str} \n}}"
 
     def visit_IntEnum(self, node):
         fields = []
@@ -494,21 +632,25 @@ class RustTranspiler(CLikeTranspiler):
             f"flags! {{\n    pub enum {node.name}: c_int {{\n{fields}\n    }}\n}}\n\n"
         )
 
-    def visit_alias(self, node):
-        return "use {0};".format(node.name)
+    def _import(self, name: str) -> str:
+        if name not in self._rust_ignored_module_set:
+            self._usings.add(name)
+        return ""
 
-    def visit_Import(self, node):
-        imports = [self.visit(n) for n in node.names]
-        return "\n".join(i for i in imports if i)
-
-    def visit_ImportFrom(self, node):
-        if node.module in self.IGNORED_MODULE_LIST:
+    def _import_from(self, module_name: str, names: List[str]) -> str:
+        if module_name in self._rust_ignored_module_set:
             return ""
-
-        names = [n.name for n in node.names]
+        self._usings.add(module_name)
+        if len(names) == 1:
+            # TODO: make this more generic so it works for len(names) > 1
+            name = names[0]
+            lookup = f"{module_name}.{name}"
+            if lookup in MODULE_DISPATCH_TABLE:
+                rust_use = MODULE_DISPATCH_TABLE[lookup]
+                return f"use {rust_use};"
+        module_name = module_name.replace(".", "::")
         names = ", ".join(names)
-        module_path = node.module.replace(".", "::")
-        return "use {0}::{{{1}}};".format(module_path, names)
+        return f"use {module_name}::{{{names}}};"
 
     def visit_List(self, node):
         self._usings.add("std::collections")
@@ -548,11 +690,19 @@ class RustTranspiler(CLikeTranspiler):
             return "{0}<{1}>".format(value, index)
         # TODO: optimize this. We need to compute value_type once per definition
         self._generic_typename_from_annotation(node.value)
-        value_type = getattr(node.value.annotation, "generic_container_type", None)
-        is_list = value_type is not None and value_type[0] == "List"
-        index_typename = get_inferred_rust_type(self._slice_value(node))
-        if is_list and (index_typename != "u64" or index_typename != "usize"):
-            index = self._cast(index, "usize")
+        if hasattr(node.value, "annotation"):
+            value_type = getattr(node.value.annotation, "generic_container_type", None)
+            is_list = value_type is not None and value_type[0] == "List"
+            if is_list:
+                index_typename = get_inferred_rust_type(self._slice_value(node))
+                if index_typename != "u64" or index_typename != "usize":
+                    index = self._cast(index, "usize")
+            is_dict = value_type is not None and value_type[0] == "Dict"
+            if is_dict:
+                value_type = getattr(node.value.annotation, "container_type", None)
+                index_typename = get_inferred_rust_type(self._slice_value(node))
+                if index_typename == value_type[1][0]:
+                    index = "&" + index
         return "{0}[{1}]".format(value, index)
 
     def visit_Index(self, node):
@@ -575,45 +725,56 @@ class RustTranspiler(CLikeTranspiler):
             return elts
         return "({0})".format(elts)
 
-    def visit_unsupported_body(self, name, body):
-        buf = ["let {0} = {{ //unsupported".format(name)]
-        buf += [self.visit(n) for n in body]
-        buf.append("};")
-        return buf
-
     def visit_Try(self, node, finallybody=None):
-        buf = self.visit_unsupported_body("try_dummy", node.body)
+        # buf = self.visit_unsupported_body(node, "try_dummy", node.body)
+        buf = ["if true {"]
+        buf += [self.visit(n) for n in node.body]
 
         for handler in node.handlers:
             buf += self.visit(handler)
         # buf.append("\n".join(excepts));
 
         if finallybody:
-            buf += self.visit_unsupported_body("finally_dummy", finallybody)
+            buf += [self.visit(n) for n in finallybody]
 
+        buf += ["}"]
         return "\n".join(buf)
 
     def visit_ExceptHandler(self, node):
         exception_type = ""
         if node.type:
             exception_type = self.visit(node.type)
-        name = "except!({0})".format(exception_type)
-        body = self.visit_unsupported_body(name, node.body)
+        name = "// except!({0})".format(exception_type)
+        body = self.visit_unsupported_body(node, name, node.body)
+        body = [self.comment(b) for b in body]
         return body
 
     def visit_Assert(self, node):
         return "assert!({0});".format(self.visit(node.test))
 
+    def _compute_kw(self, node, target) -> str:
+        kw = "let"
+        mut = is_mutable(node.scopes, get_id(target))
+        if is_global(node) or getattr(node, "class_assignment", False):
+            # Note that static are not really supported, as modifying them requires adding
+            # "unsafe" blocks, which pyrs does not do.
+            kw = "pub static" if mut else "pub const"
+        elif mut:
+            kw = "let mut"
+        return kw
+
     def visit_AnnAssign(self, node):
         target, type_str, val = super().visit_AnnAssign(node)
-        mut = "mut " if is_mutable(node.scopes, get_id(node.target)) else ""
-        return f"let {mut}{target}: {type_str} = {val};"
+        kw = self._compute_kw(node, node.target)
+        return f"{kw} {target}: {type_str} = {val};"
 
     def _needs_cast(self, left, right) -> bool:
         if not hasattr(left, "annotation") or not hasattr(right, "annotation"):
             return False
         left_type = self._typename_from_annotation(left)
-        right_type = get_inferred_rust_type(right)
+        # populate right.rust_annotation
+        get_inferred_rust_type(right)
+        right_type = self._typename_from_annotation(right)
         return left_type != right_type and left_type != self._default_type
 
     def _assign_cast(
@@ -637,19 +798,7 @@ class RustTranspiler(CLikeTranspiler):
         return f"{target_str} {op}= {value};"
 
     def _visit_AssignOne(self, node, target):
-        kw = "let"
-        mut = is_mutable(node.scopes, get_id(target))
-        if is_global(node):
-            # Note that static are not really supported, as modifying them requires adding
-            # "unsafe" blocks, which pyrs does not do.
-            kw = "pub static" if mut else "pub const"
-        elif mut:
-            kw = "let mut"
-
-        if isinstance(target, ast.Tuple):
-            elts = ", ".join([self.visit(e) for e in target.elts])
-            value = self.visit(node.value)
-            return f"{kw} ({elts}) = {value};"
+        kw = self._compute_kw(node, target)
 
         if isinstance(node.scopes[-1], ast.If):
             outer_if = node.scopes[-1]
@@ -665,7 +814,9 @@ class RustTranspiler(CLikeTranspiler):
                 value = "None"
             return "{0} = {1};".format(target, value)
 
-        definition = node.scopes.find(target.id)
+        definition = node.scopes.parent_scopes.find(get_id(target))
+        if definition is None:
+            definition = node.scopes.find(get_id(target))
         if isinstance(target, ast.Name) and defined_before(definition, node):
             needs_cast = self._needs_cast(target, node.value)
             target_str = self.visit(target)
@@ -676,66 +827,44 @@ class RustTranspiler(CLikeTranspiler):
                     value, target_type, target.annotation, node.value.rust_annotation
                 )
             return f"{target_str} = {value};"
-        elif isinstance(node.value, ast.List):
+        elif isinstance(node.value, ast.List) and kw.startswith("pub "):
             count = len(node.value.elts)
             target = self.visit(target)
             value = self.visit(node.value)
-            typename = self._typename_from_annotation(node.value)
-
-            if kw.startswith("pub "):
-                # Use arrays instead of Vec as globals must have fixed size
-                if value.startswith("vec!"):
-                    value = value.replace("vec!", "&")
-                element_type = self._default_type
-                if hasattr(node.value, "container_type"):
-                    container_type, element_type = node.value.container_type
-                return f"{kw} {target}: &[{element_type}; {count}] = {value};"
-
-            mut = "mut " if is_mutable(node.scopes, target) else ""
+            # populate node.value.container_type
+            self._typename_from_annotation(node.value)
+            # Use arrays instead of Vec as globals must have fixed size
+            if value.startswith("vec!"):
+                value = value.replace("vec!", "&")
+            element_type = self._default_type
             if hasattr(node.value, "container_type"):
-                return f"{kw} {target}: &{mut}{typename} = &{mut}{value};"
-
-            return f"{kw} {target}: {typename} = {value};"
-        elif isinstance(node.value, ast.Set):
+                container_type, element_type = node.value.container_type
+            return f"{kw} {target}: &[{element_type}; {count}] = {value};"
+        elif isinstance(node.value, ast.Set) and kw.startswith("pub "):
             target = self.visit(target)
             value = self.visit(node.value)
             typename = self._typename_from_annotation(node.value)
 
-            if kw.startswith("pub "):
-                self._usings.add("lazy_static::lazy_static")
-                if "str" in typename:
-                    typename = typename.replace("str", "'static str")
-                return (
-                    f"lazy_static! {{ pub static ref {target}: {typename} = {value}; }}"
-                )
-
-            mut = "mut " if is_mutable(node.scopes, target) else ""
-            if hasattr(node.value, "container_type"):
-                return f"{kw} {target}: &{mut}{typename} = &{mut}{value};"
-
-            return f"{kw} {target}: {typename} = {value};"
-        elif isinstance(node.value, ast.Dict):
+            self._usings.add("lazy_static::lazy_static")
+            if "str" in typename:
+                typename = typename.replace("str", "'static str")
+            return f"lazy_static! {{ pub static ref {target}: {typename} = {value}; }}"
+        elif isinstance(node.value, ast.Dict) and kw.startswith("pub "):
             target = self.visit(target)
             value = self.visit(node.value)
             typename = self._typename_from_annotation(node.value)
 
-            if kw.startswith("pub "):
-                if hasattr(node.value, "container_type"):
-                    container_type, element_type = node.value.container_type
-                    key_typename, value_typename = element_type
-                    if key_typename == "&str":
-                        key_typename = "&'static str"
-                    if value_typename == "&str":
-                        value_typename = "&'static str"
-                    typename = f"{key_typename}, {value_typename}"
-
-                return f"lazy_static! {{ pub static ref {target}: HashMap<{typename}> = {value}; }}"
-
-            mut = "mut " if is_mutable(node.scopes, target) else ""
+            self._usings.add("lazy_static::lazy_static")
             if hasattr(node.value, "container_type"):
-                return f"{kw} {target}: &{mut}{typename} = &{mut}{value};"
+                container_type, element_type = node.value.container_type
+                key_typename, value_typename = element_type
+                if key_typename == "&str":
+                    key_typename = "&'static str"
+                if value_typename == "&str":
+                    value_typename = "&'static str"
+                typename = f"{key_typename}, {value_typename}"
 
-            return f"{kw} {target}: {typename} = {value};"
+            return f"lazy_static! {{ pub static ref {target}: HashMap<{typename}> = {value}; }}"
         else:
             typename = self._typename_from_annotation(target)
             needs_cast = self._needs_cast(target, node.value)
@@ -745,7 +874,14 @@ class RustTranspiler(CLikeTranspiler):
                 value = self._assign_cast(
                     value, typename, target.annotation, node.value.annotation
                 )
-            return f"{kw} {target_str}: {typename} = {value};"
+            if hasattr(node.value, "container_type"):
+                mut = "mut " if is_mutable(node.scopes, target_str) else ""
+                typename = f"&{mut}{typename}"
+                value = f"&{mut}{value}"
+            optional_typename = (
+                f": {typename}" if typename != self._default_type else ""
+            )
+            return f"{kw} {target_str}{optional_typename} = {value};"
 
     def visit_Delete(self, node):
         target = node.targets[0]
@@ -759,27 +895,6 @@ class RustTranspiler(CLikeTranspiler):
         # specifying the exception.
         return "raise!(); //unsupported"
 
-    def visit_With(self, node):
-        buf = []
-
-        with_statement = "// with!("
-        for i in node.items:
-            if i.optional_vars:
-                with_statement += "{0} as {1}, ".format(
-                    self.visit(i.context_expr), self.visit(i.optional_vars)
-                )
-            else:
-                with_statement += "{0}, ".format(self.visit(i.context_expr))
-        with_statement = with_statement[:-2] + ") //unsupported\n{"
-        buf.append(with_statement)
-
-        for n in node.body:
-            buf.append(self.visit(n))
-
-            buf.append("}")
-
-        return "\n".join(buf)
-
     def visit_Await(self, node):
         value = self.visit(node.value)
         return f"{value}.await"
@@ -792,8 +907,11 @@ class RustTranspiler(CLikeTranspiler):
         self._features.add("generator_trait")
         self._usings.add("std::ops::Generator")
         self._usings.add("std::ops::GeneratorState")
-        value = self.visit(node.value)
-        return f"yield {value};"
+        if node.value is not None:
+            value = self.visit(node.value)
+            return f"yield {value};"
+        else:
+            return "yield None;"
 
     def visit_Print(self, node):
         buf = []
@@ -802,9 +920,6 @@ class RustTranspiler(CLikeTranspiler):
             buf.append('println!("{{:?}}",{0});'.format(value))
         return "\n".join(buf)
 
-    def visit_DictComp(self, node):
-        return "DictComp /*unimplemented()*/"
-
     def visit_GeneratorExp(self, node):
         elt = self.visit(node.elt)
         generator = node.generators[0]
@@ -813,7 +928,7 @@ class RustTranspiler(CLikeTranspiler):
 
         is_range = (
             ("range" in get_id(generator.iter.func))
-            if isinstance(generator.iter, ast.Call)
+            if isinstance(generator.iter, ast.Call) and get_id(generator.iter.func)
             else False
         )
         # HACK for dictionary iterators to work
