@@ -1,10 +1,6 @@
 from typing import List
 
 
-def do_pass():
-    pass
-
-
 def inline_pass():
     pass
 
@@ -61,8 +57,8 @@ def show():
     print(len(a5))
     a9: List[str] = ["a", "b", "c", "d"]
     print(len(a9))
-    a6 = {1, 2, 3, 4}
-    print(len(a6))
+    # a6 = {1, 2, 3, 4}
+    # print(len(a6))
     a7 = {"a": 1, "b": 2}
     print(len(a7))
     a8 = True
@@ -77,15 +73,23 @@ def show():
         print("true")
     if 1 != None:
         print("World is sane")
-    # print(True)
-    # if True: a1 += 1
-    # if True: print("true")
-    do_pass()
+    print(True)
+    if True:
+        a1 += 1
+    assert a1 == 11
+    if True:
+        print("true")
     inline_pass()
     s = "1\
     2"
     print(s)
+    assert infer_bool(1)
     # assert 1 != 2 != 3
+    # Escape quotes
+    _escape_quotes = """ foo "bar" baz """
+    assert "bbc" in "aaabbccc"
+    assert bool(1)
+    (_c1, _, _c2) = (1, 2, 3)
 
 
 if __name__ == "__main__":
