@@ -15,7 +15,7 @@ fun bubble_sort(seq: Array<Int>): Array<Int> {
     return seq
 }
 
-fun main() {
+fun main(argv: Array<String>) {
     var unsorted = arrayOf(14, 11, 19, 5, 16, 10, 19, 12, 5, 12)
     val expected = arrayOf(5, 5, 10, 11, 12, 12, 14, 16, 19, 19)
     assert(bubble_sort(unsorted) == expected)
