@@ -1,11 +1,13 @@
-#include "py14/runtime/builtins.h"
-#include "py14/runtime/range.hpp"
-#include "py14/runtime/sys.h"
-#include <cassert>
-#include <iostream>
-#include <math.h>
+#include <math.h>  // NOLINT(build/include_order)
+
+#include <cassert>   // NOLINT(build/include_order)
+#include <iostream>  // NOLINT(build/include_order)
 #include <tuple>
-#include <vector>
+#include <vector>  // NOLINT(build/include_order)
+
+#include "pycpp/runtime/builtins.h"  // NOLINT(build/include_order)
+#include "pycpp/runtime/range.hpp"   // NOLINT(build/include_order)
+#include "pycpp/runtime/sys.h"       // NOLINT(build/include_order)
 
 inline std::vector<int> comb_sort(std::vector<int> seq) {
   auto gap = seq.size();
@@ -24,8 +26,8 @@ inline std::vector<int> comb_sort(std::vector<int> seq) {
   return seq;
 }
 
-int main(int argc, char **argv) {
-  py14::sys::argv = std::vector<std::string>(argv, argv + argc);
+int main(int argc, char** argv) {
+  pycpp::sys::argv = std::vector<std::string>(argv, argv + argc);
   std::vector<int> unsorted = {14, 11, 19, 5, 16, 10, 19, 12, 5, 12};
   std::vector<int> expected = {5, 5, 10, 11, 12, 12, 14, 16, 19, 19};
   assert(comb_sort(unsorted) == expected);
