@@ -8,7 +8,6 @@ int fib(int i) {
   return (fib((i - 1)) + fib((i - 2)));
 }
 
-main() {
-  final int rv = fib(5);
-  print(sprintf("%s", [rv]));
+main(List<String> argv) {
+  print(sprintf("%s", [fib(5)]));
 }
