@@ -6,8 +6,7 @@ function fib(i::Int64)::Int64
 end
 
 function main()
-    rv = fib(5)
-    println(join([rv], " "))
+    println(join([fib(5)], " "))
 end
 
 main()
