@@ -5,7 +5,9 @@ fun fib(i: Int): Int {
     return (fib((i - 1)) + fib((i - 2)))
 }
 
-fun main() {
-    val rv = fib(5)
-    println("$rv")
+fun main(argv: Array<String>) {
+    if (true) {
+        val __tmp1 = fib(5)
+        println("$__tmp1")
+    }
 }
