@@ -1,7 +1,8 @@
-#include "py14/runtime/builtins.h"
-#include "py14/runtime/sys.h"
-#include <cassert>
-#include <iostream>
+#include <cassert>   // NOLINT(build/include_order)
+#include <iostream>  // NOLINT(build/include_order)
+
+#include "pycpp/runtime/builtins.h"  // NOLINT(build/include_order)
+#include "pycpp/runtime/sys.h"       // NOLINT(build/include_order)
 
 inline void foo() {
   int a = 10;
@@ -45,14 +46,13 @@ inline int16_t mul(int8_t x, int8_t y) { return x * y; }
 inline double fadd1(int8_t x, double y) { return x + y; }
 
 inline void show() {
-  double rv = fadd1(6, 6.0);
-  assert(rv == 12);
+  assert(fadd1(6, 6.0) == 12);
   std::cout << std::string{"OK"};
   std::cout << std::endl;
 }
 
-int main(int argc, char **argv) {
-  py14::sys::argv = std::vector<std::string>(argv, argv + argc);
+int main(int argc, char** argv) {
+  pycpp::sys::argv = std::vector<std::string>(argv, argv + argc);
   foo();
   show();
 }
