@@ -67,12 +67,11 @@ double fadd1(int x, double y) {
 }
 
 show() {
-  final double rv = fadd1(6, 6.0);
-  assert(rv == 12);
+  assert(fadd1(6, 6.0) == 12);
   print(sprintf("%s", ["OK"]));
 }
 
-main() {
+main(List<String> argv) {
   foo();
   show();
 }
