@@ -1,8 +1,9 @@
-#include "py14/runtime/builtins.h"
-#include "py14/runtime/sys.h"
-#include <cassert>
-#include <iostream>
-#include <map>
+#include <cassert>   // NOLINT(build/include_order)
+#include <iostream>  // NOLINT(build/include_order)
+#include <map>       // NOLINT(build/include_order)
+
+#include "pycpp/runtime/builtins.h"  // NOLINT(build/include_order)
+#include "pycpp/runtime/sys.h"       // NOLINT(build/include_order)
 
 enum Colors : int {
   RED = 0,
@@ -40,7 +41,7 @@ inline void show() {
   assert(color_map.size() != 0);
 }
 
-int main(int argc, char **argv) {
-  py14::sys::argv = std::vector<std::string>(argv, argv + argc);
+int main(int argc, char** argv) {
+  pycpp::sys::argv = std::vector<std::string>(argv, argv + argc);
   show();
 }
