@@ -1,12 +1,21 @@
-
 //! ```cargo
 //! [package]
 //! edition = "2018"
 //! [dependencies]
+//! anyhow = "*"
 //! flagset = "*"
 //! ```
 
+#![allow(clippy::collapsible_else_if)]
+#![allow(clippy::double_parens)] // https://github.com/adsharma/py2many/issues/17
+#![allow(clippy::map_identity)]
+#![allow(clippy::needless_return)]
+#![allow(clippy::print_literal)]
+#![allow(clippy::ptr_arg)]
+#![allow(clippy::redundant_static_lifetimes)] // https://github.com/adsharma/py2many/issues/266
+#![allow(clippy::unnecessary_cast)]
 #![allow(clippy::upper_case_acronyms)]
+#![allow(clippy::useless_vec)]
 #![allow(non_camel_case_types)]
 #![allow(non_snake_case)]
 #![allow(non_upper_case_globals)]
@@ -14,7 +23,9 @@
 #![allow(unused_mut)]
 #![allow(unused_parens)]
 
+extern crate anyhow;
 extern crate flagset;
+use anyhow::Result;
 use flagset::flags;
 use std::collections::HashMap;
 use std::os::raw::c_int;
@@ -55,9 +66,10 @@ pub fn show() {
     } else {
         println!("{}", "Not R");
     }
-    assert!(color_map.len() as i32 != 0);
+    assert!((color_map.len() as i32) != 0);
 }
 
-pub fn main() {
+pub fn main() -> Result<()> {
     show();
+    Ok(())
 }
