@@ -7,6 +7,6 @@ fun show() {
     }
 }
 
-fun main() {
+fun main(argv: Array<String>) {
     show()
 }
