@@ -1,6 +1,7 @@
-#include "py14/runtime/builtins.h"
-#include "py14/runtime/sys.h"
-#include <iostream>
+#include <iostream>  // NOLINT(build/include_order)
+
+#include "pycpp/runtime/builtins.h"  // NOLINT(build/include_order)
+#include "pycpp/runtime/sys.h"       // NOLINT(build/include_order)
 inline void show() {
   std::cout << std::string{"b"};
   std::cout << std::endl;
@@ -19,7 +20,7 @@ inline void show() {
   std::cout << std::endl;
 }
 
-int main(int argc, char **argv) {
-  py14::sys::argv = std::vector<std::string>(argv, argv + argc);
+int main(int argc, char** argv) {
+  pycpp::sys::argv = std::vector<std::string>(argv, argv + argc);
   show();
 }
