@@ -1,11 +1,12 @@
-#include "py14/runtime/builtins.h"
-#include "py14/runtime/sys.h"
-#include <cassert>
-#include <iostream>
+#include <cassert>   // NOLINT(build/include_order)
+#include <iostream>  // NOLINT(build/include_order)
+
+#include "pycpp/runtime/builtins.h"  // NOLINT(build/include_order)
+#include "pycpp/runtime/sys.h"       // NOLINT(build/include_order)
 /* This file implements a rectangle class  */
 
 class Rectangle {
-public:
+ public:
   int height;
   int length;
 
@@ -27,7 +28,7 @@ inline void show() {
   std::cout << std::endl;
 }
 
-int main(int argc, char **argv) {
-  py14::sys::argv = std::vector<std::string>(argv, argv + argc);
+int main(int argc, char** argv) {
+  pycpp::sys::argv = std::vector<std::string>(argv, argv + argc);
   show();
 }
