@@ -1,12 +1,20 @@
-
 //! ```cargo
 //! [package]
 //! edition = "2018"
 //! [dependencies]
-//!
+//! anyhow = "*"
 //! ```
 
+#![allow(clippy::collapsible_else_if)]
+#![allow(clippy::double_parens)] // https://github.com/adsharma/py2many/issues/17
+#![allow(clippy::map_identity)]
+#![allow(clippy::needless_return)]
+#![allow(clippy::print_literal)]
+#![allow(clippy::ptr_arg)]
+#![allow(clippy::redundant_static_lifetimes)] // https://github.com/adsharma/py2many/issues/266
+#![allow(clippy::unnecessary_cast)]
 #![allow(clippy::upper_case_acronyms)]
+#![allow(clippy::useless_vec)]
 #![allow(non_camel_case_types)]
 #![allow(non_snake_case)]
 #![allow(non_upper_case_globals)]
@@ -14,6 +22,9 @@
 #![allow(unused_mut)]
 #![allow(unused_parens)]
 
+extern crate anyhow;
+use anyhow::Result;
+
 /* This file implements a rectangle class  */
 
 pub struct Rectangle {
@@ -41,6 +52,7 @@ pub fn show() {
     println!("{}", r.length);
 }
 
-pub fn main() {
+pub fn main() -> Result<()> {
     show();
+    Ok(())
 }
