@@ -1,11 +1,12 @@
-#include "py14/runtime/builtins.h"
-#include "py14/runtime/sys.h"
-#include <iostream>
-#include <map>
+#include <iostream>  // NOLINT(build/include_order)
+#include <map>       // NOLINT(build/include_order)
+
+#include "pycpp/runtime/builtins.h"  // NOLINT(build/include_order)
+#include "pycpp/runtime/sys.h"       // NOLINT(build/include_order)
 
 class Colors : public std::string {
-public:
-  Colors(const char *s) : std::string(s) {}
+ public:
+  Colors(const char* s) : std::string(s) {}  // NOLINT(runtime/explicit)
   static const Colors RED;
   static const Colors GREEN;
   static const Colors BLUE;
@@ -32,7 +33,7 @@ inline void show() {
   std::cout << std::endl;
 }
 
-int main(int argc, char **argv) {
-  py14::sys::argv = std::vector<std::string>(argv, argv + argc);
+int main(int argc, char** argv) {
+  pycpp::sys::argv = std::vector<std::string>(argv, argv + argc);
   show();
 }
