@@ -1,13 +1,22 @@
-
 //! ```cargo
 //! [package]
 //! edition = "2018"
 //! [dependencies]
+//! anyhow = "*"
 //! strum = "*"
 //! strum_macros = "*"
 //! ```
 
+#![allow(clippy::collapsible_else_if)]
+#![allow(clippy::double_parens)] // https://github.com/adsharma/py2many/issues/17
+#![allow(clippy::map_identity)]
+#![allow(clippy::needless_return)]
+#![allow(clippy::print_literal)]
+#![allow(clippy::ptr_arg)]
+#![allow(clippy::redundant_static_lifetimes)] // https://github.com/adsharma/py2many/issues/266
+#![allow(clippy::unnecessary_cast)]
 #![allow(clippy::upper_case_acronyms)]
+#![allow(clippy::useless_vec)]
 #![allow(non_camel_case_types)]
 #![allow(non_snake_case)]
 #![allow(non_upper_case_globals)]
@@ -15,8 +24,10 @@
 #![allow(unused_mut)]
 #![allow(unused_parens)]
 
+extern crate anyhow;
 extern crate strum;
 extern crate strum_macros;
+use anyhow::Result;
 use std::collections::HashMap;
 use strum_macros::{Display, EnumString, EnumVariantNames};
 
@@ -48,6 +59,7 @@ pub fn show() {
     println!("{}", color_map.len());
 }
 
-pub fn main() {
+pub fn main() -> Result<()> {
     show();
+    Ok(())
 }
